/* Copyright (c) 2000-2002 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

// Note that in general, we use ui_state instead of the debugger state
// whenever we must handle "running" _and_ "debugging" - when running
// (but not debugging) only ui_state will have changed.

// Note that we can never use the "current file's extension" or other
// simple rules to determine "can we debug".  *ANY* file may have a 
// valid invocation setup.  So for now, this code assumes that
// ALL files are debuggable - this will mean that ALL files prompt
// for an invocation, which the user may cancel if they made
// an error.

function debugger_updateCommandset(commandset) {
    DBG.controller.update_state();
    command_updateCommandset(commandset)
}

// Execute a "start debugger" command - 
// the "Control" key switches whether to show the dialog
// or not, with the default being set by the dbgShowDialog pref
function debugger_doCommand(command, event)
{
    DBG.controller.ctrlKeyForStart = event.ctrlKey;
    return command_doCommand(command)
}

function DebuggerItemsController() {
    this.ctrlKeyForStart = 0;
    window.controllers.appendController(this);
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
DebuggerItemsController.prototype = new Controller();
DebuggerItemsController.prototype.constructor = DebuggerItemsController;

DebuggerItemsController.prototype.destroy = function() {
    window.controllers.removeController(this);
}

DebuggerItemsController.prototype.update_state = function() {
    try {
        this.DBG_STATE = DBG.session.dbg.state;
    } catch(e) {
        this.DBG_STATE = DBG.STATE_STOPPED;
    }
}

DebuggerItemsController.prototype.is_cmd_dbgStepIn_enabled = function () {
    return DBG.session.canStep();
}

DebuggerItemsController.prototype.do_cmd_dbgStepIn = function() {
    return debugger_do_step(Components.interfaces.koIDebugger.RESUME_STEP_IN);
}

DebuggerItemsController.prototype.is_cmd_dbgStepOver_enabled = function () {
    return DBG.session.canStep();
}

DebuggerItemsController.prototype.do_cmd_dbgStepOver = function() {
    return debugger_do_step(Components.interfaces.koIDebugger.RESUME_STEP_OVER);
}

DebuggerItemsController.prototype.is_cmd_dbgStepOut_enabled = function () {
    return DBG.session.canStep();
}

DebuggerItemsController.prototype.do_cmd_dbgStepOut = function() {
    return debugger_do_step(Components.interfaces.koIDebugger.RESUME_STEP_OUT);
}

DebuggerItemsController.prototype.is_cmd_dbgStop_enabled = function () {
    return this.DBG_STATE != DBG.STATE_STOPPED;
}

DebuggerItemsController.prototype.do_cmd_dbgStop = function() {
    return DBG.session.terminate();
}
DebuggerItemsController.prototype.is_cmd_dbgStepOut_enabled = function () {
    return DBG.session.canStep();
}
DebuggerItemsController.prototype.is_cmd_dbgGo_enabled = function () {
    return DBG.session.canStep();
}
DebuggerItemsController.prototype.do_cmd_dbgGo = function() {
    return debugger_do_step(Components.interfaces.koIDebugger.RESUME_GO);
}

DebuggerItemsController.prototype.is_cmd_dbgRun_enabled = function () {
    return DBG.session.canRun();
}
DebuggerItemsController.prototype.do_cmd_dbgRun = function() {
    return debugger_run_script();
}

DebuggerItemsController.prototype.is_cmd_dbgShowCurrentStatement_enabled = function () {
    return this.DBG_STATE == DBG.STATE_BREAK;
}
DebuggerItemsController.prototype.do_cmd_dbgShowCurrentStatement = function() {
    debugger_show_current_statement();
}
DebuggerItemsController.prototype .is_cmd_dbgBreakpointToggle_enabled = function () {
    return (gViewMgr.currentView &&
            gViewMgr.currentView.document &&
            gViewMgr.currentView.getAttribute('type') == 'editor');  // skip browser views
            // XXX we need to allow breakpoints to be set in XML files
            //&& gViewMgr.currentView.document.languageObj.debuggable);
}
DebuggerItemsController.prototype.do_cmd_dbgBreakpointToggle = function() {
      return debugger_toggle_breakpoint();
}
DebuggerItemsController.prototype .is_cmd_dbgBreakpointClearAll_enabled = function () {
    return (gViewMgr.currentView &&
            gViewMgr.currentView.document &&
            gViewMgr.currentView.getAttribute('type') == 'editor');  // skip browser views
            // XXX we need to allow breakpoints to be set in XML files
            //&& gViewMgr.currentView.document.languageObj.debuggable);
}
DebuggerItemsController.prototype.do_cmd_dbgBreakpointClearAll = function() {
      return debugger_clear_breakpoints();
}
DebuggerItemsController.prototype .is_cmd_dbgListen_enabled = function () {
    return this.DBG_STATE == DBG.STATE_STOPPED;
}
DebuggerItemsController.prototype.do_cmd_dbgListen = function() {
    if (DBG.listener_manager.isListening()) return false;
    return DBG.listener_manager.startListening(0, true);
}
// supported handles the greyed out of the menu item
DebuggerItemsController.prototype.is_cmd_debuggerListener_supported = function() {
    return this.DBG_STATE == DBG.STATE_STOPPED;
}
// enabled is the checkmark
DebuggerItemsController.prototype.is_cmd_debuggerListener_enabled = function() {
    // pref listenForDebugger
    return DBG.listener_manager.isListening();
}
// called on changing the state of the menu checkmark
DebuggerItemsController.prototype.do_cmd_debuggerListener = function() {
    // listenfordebugger pref is for starting listening when komodo starts
    if (DBG.listener_manager.isListening()) {
        DBG.listener_manager.stopListening();
        globalPrefs.setBooleanPref("listenForDebugger",false);
    } else {
        DBG.listener_manager.startListening(0, false);
        globalPrefs.setBooleanPref("listenForDebugger",true);
    }
}
