/* Copyright (c) 2000-2003 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

/* Komodo startup/shutdown handling.
 *
 * This file should ONLY contain functionality directly relevant to startup and
 * shutdown. Enforcer: ShaneC.
 */


// Some parts of the InitService must run before almost ANYTHING else in the
// Komodo system. Putting this at the top of KomodoOnLoad() is often not enough
// because some .js files included by jaguar.xul will do some work with
// top-level code. Putting this here and ensuring that this is first .js file
// included should guarantee that this is run first.
function KomodoPlatformInit() {
    var initSvc = Components.classes["@activestate.com/koInitService;1"].
                  getService(Components.interfaces.koIInitService);
    try {
        initSvc.setPlatformErrorMode();
        initSvc.setEncoding();
        initSvc.initProcessUtils();
        initSvc.upgradeUserSettings();
    } catch(e) {
        log.exception(e,"***Komodo Initialization Service failed***");
        initSvc = null;
    }
}
KomodoPlatformInit();


var gPrefSvc = null;
var gPrefs = null;
var gInfoSvc = null;
var _gCommandmentSvc = null;

// XXX don't use this for new code, it's here to support
// older code
var gObserverSvc = null;

function KomodoPrefsInit() {
    gPrefSvc = Components.classes["@activestate.com/koPrefService;1"].
                    getService(Components.interfaces.koIPrefService);
    gPrefs = gPrefSvc.prefs;
    KomodoRegisterOnUnloadHandler(KomodoPrefsOnClose);
}

function KomodoPrefsOnClose() {
    gPrefSvc.shutDown();
}


function KomodoServicesInit()
{
    gPrefSvc.getPrefs("viewStateMRU").max_length = gPrefs.getLongPref("viewStateMRUSize");
    gPrefSvc.getPrefs("docStateMRU").max_length = gPrefs.getLongPref("docStateMRUSize");

    var templateSvc = Components.classes["@activestate.com/koTemplateService;1"].getService();
    try {
        templateSvc.initializeUserTemplateTree();
    } catch(ex) {
        var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
                            getService(Components.interfaces.koILastErrorService);
        var errmsg = lastErrorSvc.getLastErrorMessage();
        alert("There was an error initializing your Komodo user "+
                     "settings directory with template information: "+errmsg+
                     ". This may mean that you will not be able to create "+
                     "your own custom templates. You will still be able "+
                     "to use Komodo's numerous standard templates.");
    }

    var featureStatusSvc = Components.classes["@activestate.com/koFeatureStatusService;1"].
            getService(Components.interfaces.koIFeatureStatusService);
    featureStatusSvc.initialize();

    gObserverSvc = Components.classes["@mozilla.org/observer-service;1"].
            getService(Components.interfaces.nsIObserverService);
    KomodoRegisterOnUnloadHandler(KomodoServicesShutdown);
}

function KomodoServicesShutdown()
{
    var featureStatusSvc = Components.classes["@activestate.com/koFeatureStatusService;1"].
            getService(Components.interfaces.koIFeatureStatusService);
    featureStatusSvc.finalize();







    var docSvc = Components.classes["@activestate.com/koDocumentService;1"].
                 getService(Components.interfaces.koIDocumentService);
    docSvc.unload(); // shut down the autosave thread
    
    // Save the view MRU (*sob* - this needs more thought -
    // really want to save all pref files known to the service at shutdown.
    gPrefSvc.savePrefsState("viewStateMRU");
}


// Check the current license validity and interact with the user accordingly.
//
//  "_mode" is an internal variable that should be used only for testing.
//
// This returns true iff the user's Komodo license is valid. Note that this
// method may involve installing a valid license as a side-effect.
//
function KomodoLicenseCheck(_mode /* =null */)
{
    if (typeof(_mode) == "undefined") _mode = null;
    var err;
    if (_mode || !gInfoSvc.licenseValid) {
        // Determine what "mode" we are in.
        var mode;
        if (_mode) {
            mode = _mode;
        } else {
            // - A build which includes a siloed non-expiring license.
            if (gInfoSvc.siloedLicense && !gInfoSvc.siloedLicenseExpires) {
                mode = "siloed non-expiring license";
            }
            // - A build which includes a siloed expiring license (presumably
            //   this is a beta) and the period has expired.
            else if (gInfoSvc.siloedLicense && gInfoSvc.siloedLicenseExpires
                     && gInfoSvc.siloedLicenseExpired)
            {
                mode = "siloed expired license";
            }
            // - A build which includes a siloed expiring license (presumably
            //   this is a beta) and the period has NOT expired.
            else if (gInfoSvc.siloedLicense && gInfoSvc.siloedLicenseExpires
                     && !gInfoSvc.siloedLicenseExpired)
            {
                mode = "siloed expiring license";
            }
            // - A non-final build withOUT a siloed license.
            else if (gInfoSvc.version.indexOf('-') != -1) {
                mode = "non-final build without license";
            }
            // - A final build withOUT a siloed license.
            else {
                mode = "final build without license";
            }
        }

        // Determine the appropriate UI for the current mode.
        // - Setting "choice" in the code block implies that no dialog
        //   should be shown, the given choice should just be assumed.
        var msg=null, buttons=[], infos=[], title=null, choice=null;
        switch (mode) {
        case "siloed non-expiring license":
            //msg = "You do not currently have a valid license installed for "+
            //      "this Komodo version. Press <Enter> to install the "+
            //      "license key that comes included with this Komodo "+
            //      "installation.";
            //buttons = ["Install License Key", "Close"];
            //title = "Install License Key";
            choice = "Install License Key";
            break;
        case "siloed expired license":
            var verInfo = "Komodo " + gInfoSvc.productType + " " +
                          gInfoSvc.version;
            msg = "The license period for this Komodo installation "+
                  "("+verInfo+") has expired.";
            buttons = ["Purchase Komodo",
                       "Download Komodo Trial",
                       "Renew ASPN Membership",
                       "Close"];
            infos = ["purchase", "aspn"];
            title = "Expired Komodo Installation";
            break;
        case "siloed expiring license":
            //msg = "You do not currently have a valid license installed for "+
            //      "this Komodo version. Press <Enter> to install the "+
            //      "license key "+
            //      "that comes included with this Komodo installation. "+
            //      "Note that this license will expire in "+
            //      gInfoSvc.siloedLicenseDaysUntilExpiration+" day(s).";
            //buttons = ["Install License Key", "Close"];
            //title = "Install License Key";
            choice = "Install License Key";
            break;
        case "non-final build without license":
            msg = "You do not currently have a valid license installed for "+
                  "this Komodo version.";
            buttons = ["Get Beta License (if available)",
                       "Download Komodo Trial",
                       "Purchase Komodo",
                       "Close"];
            infos = ["purchase",



                     "beta"];
            break;
        case "final build without license":
            msg = "You do not currently have a valid license installed for "+
                  "this Komodo version.";
            buttons = ["Download Komodo Trial",
                       "Purchase Komodo",
                       "Renew ASPN Membership",
                       "Close"];
            infos = ["purchase",



                     "aspn"];
            break;
        default:
            err = "unexpected invalid license mode: '"+mode+"'\n";
            dialog_internalError(err, err);
            return false;
        }

        // Ask the user.
        if (!choice) {
            choice = dialog_invalidLicense(msg, buttons, infos, null, title);
        }

        // Handle the user's choice.
        switch(choice) {
        case "Install License Key":
            // run license installer
            var osPathSvc = Components.classes["@activestate.com/koOsPath;1"].getService();
            var koDirSvc = Components.classes["@activestate.com/koDirs;1"].getService();
            var runSvc = Components.classes["@activestate.com/koRunService;1"].getService();



            var instlic = osPathSvc.join(koDirSvc.installDir, "instlic");

            var cmd = '"'+instlic+'" -s'; // run silently
            var outputObj = new Object();
            var errorObj = new Object();
            var retval = runSvc.RunAndCaptureOutput(cmd,
                                                    null, // cwd
                                                    null, // env
                                                    null, // input
                                                    outputObj, errorObj);
            if (retval) {
                err = "Unexpected error installing license key.";
                dialog_internalError(err, err+"\n\ncmd: "+cmd+
                                     "\n\noutput:\n"+outputObj.value+
                                     "\n\nerror:\n"+errorObj.value);
            }
            
            gInfoSvc.refreshLicenseInfo();
            break;
        case "Get Beta License (if available)":
            browse_OpenUrlInDefaultBrowser("http://www.ActiveState.com/Komodo/Beta");
            break;
        case "Purchase Komodo":
            browse_OpenUrlInDefaultBrowser("http://www.ActiveState.com/Komodo/Purchase");
            break;
        case "Download Komodo Trial":
            browse_OpenUrlInDefaultBrowser("http://www.ActiveState.com/Komodo/Trial");
            break;
        case "Renew ASPN Membership":
            browse_OpenUrlInDefaultBrowser("http://www.ActiveState.com/renewal_info");
            break;
        case "Close":
            // do nothing
            break;
        default:
            err = "unexpected invalid license choice: '"+choice+"'\n";
            dialog_internalError(err, err);
            return false;
        }

        if (gInfoSvc.licenseValid) {
            return true;
        } else {
            window.close();
            return false;
        }
    }
    
    // If this license expires within a week and have not yet warned today,
    // then put up a dialog explaining when this will expire.
    else {
        var daysLeft = gInfoSvc.daysUntilExpiration;
        if (daysLeft < 0) daysLeft = -daysLeft;
        var warnDay = gPrefs.getLongPref("days_until_expiration_to_warn_2.5.0-beta1");
        if (daysLeft <= warnDay) {
            dialog_alert("Your Komodo license will expire in "+daysLeft+" day(s).\n",
                         null, // text
                         null, // title
                         "warn_about_license_expiration_2.5.0-beta1");
            gPrefs.setLongPref("days_until_expiration_to_warn_2.5.0-beta1",
                               daysLeft-1);
        }
    }
    
    return true;
}

//
// This observer handles any notification pertinent
// to komodo the application (ie. startup/shutdown issues)
// that have no other place to be handled.
//
var _gKomodoObserver = null;
function _KomodoObserver() {
    KomodoRegisterOnUnloadHandler(this.shutdown, this);
    this.startup();
};
_KomodoObserver.prototype.constructor = _KomodoObserver;
_KomodoObserver.prototype.QueryInterface = function (iid) {
    if (!iid.equals(Components.interfaces.nsIObserver) &&
        !iid.equals(Components.interfaces.nsISupports)) {
        throw Components.results.NS_ERROR_NO_INTERFACE;
    }
    return this;
}
_KomodoObserver.prototype.startup = function()
{
    var observerSvc = Components.classes["@mozilla.org/observer-service;1"].
                    getService(Components.interfaces.nsIObserverService);
    observerSvc.addObserver(this, "quit",false);
}
_KomodoObserver.prototype.shutdown = function()
{
    var observerSvc = Components.classes["@mozilla.org/observer-service;1"].
                    getService(Components.interfaces.nsIObserverService);
    try {
        observerSvc.removeObserver(this, "quit");
    } catch(e) { /* moz already removed them */ log.debug('quit '+e); }
    _gKomodoObserver = null;
}
_KomodoObserver.prototype.observe = function(subject, topic, data)
{
    //log.debug("_KomodoObserver: observed '"+topic+"' notification: data='"+data+"'\n");
    switch (topic) {
    case 'quit':
        window.setTimeout("KomodoGoQuitApplication()", 0);
        break;
    }
}

function KomodoOnLoad() {
    try {
        trace_onload();













        KomodoPrefsInit();

        var initSvc = Components.classes["@activestate.com/koInitService;1"].
                      getService(Components.interfaces.koIInitService);
        var locale = initSvc.getStartupLocale();
        if (locale &&
            gPrefs.hasPref("encodingEnvironment") &&
            gPrefs.getBooleanPref("encodingEnvironment")) {
            var enc = locale.split('.')[1].toLowerCase();
            gPrefs.setStringPref("encodingDefault", enc)
        }
        
        // simple globals that do not need a shutdown
        gInfoSvc = Components.classes["@activestate.com/koInfoService;1"].
              getService(Components.interfaces.koIInfoService);
        
        /* services needed for even the most basic operation of komodo */
        windowManager_init()
        keybindingManagerOnLoad();
        
        /* shutdown observer */
        _gKomodoObserver = new _KomodoObserver();

        var ok = KomodoLicenseCheck();
        if (!ok) return;
        
        KomodoServicesInit();
        MRU_initialize();
        
        views_onload();
        command_onload();



        StatusBar_Init();
        isearch_onload();
        projectManagerOnLoad();
        RunOutput_Init();
        // KomboOnLoad();
        // get_debugger_interface(); // start debug engines!
        FindResultsTab_OnLoad();
        toolboxManagerOnLoad();
        
        uilayout_onload();
        lint_initialize();
        debugger_onLoad();



        // anything that we want to do user interaction with at
        // startup should go into this timeout to let the window
        // onload handler finish.
        window.setTimeout("KomodoOnLoadDelay();",500);
    } catch (e) {
        log.exception(e,"Error doing KomodoOnLoad:");
        throw e;
    }
}

/* generaly this holds anything that requires user interaction
   on startup.  We want that stuff to happen after komodo's main
   windows is up and running.  There may be a thing or two otherwise
   that also needs to start late. */
function KomodoOnLoadDelay() {

    // Used by perf_timeline.perf_startup. Mark before
    // commandmentSvc.initialize() because that will immediately start
    // executing queued up commandments.
    gTimeline.mark("startup complete");

    // initialize the js dialog proxy with our window so that we can
    // avoid using the hidden window
    try {
        var dialogproxy = Components.classes['@activestate.com/asDialogProxy;1'].
                            getService(Components.interfaces.asIDialogProxy);
        dialogproxy.init(window);
    } catch (e) {
        log.exception(e);
    }
    
    // Openning the Start Page should be before commandment system init and
    // workspace restoration because it should be the first view opened.
    if (gPrefs.getBooleanPref("show_start_page")) {
        open_openStartPage();
    }

    /* commandmentSvc needs the window to be alive */
    _gCommandmentSvc = Components.classes["@activestate.com/koCommandmentService;1"]
                       .getService(Components.interfaces.koICommandmentService);
    _gCommandmentSvc.initialize();
    
    SCCWarnAboutSSHIfNecessary();

    // the offer to restore the workspace needs to be after the
    // commandments system is initialized because the commandments mechanism
    // is how the determination of 'running in non-interactive mode' happens,
    // which the restoration step needs to know about.
    view_restoreWorkspace(); 

    // Possibly show the "Tip of the Day" dialog.
    if (gPrefs.getBooleanPref("tipShowAtStartup")) {
       launch_TipOfTheDay();
    }











    KomodoRegisterOnUnloadHandler(KomodoUnLoadDelayItems);
    KomodoRegisterOnUnloadHandler(Komodo_CloseSubwindows);
}

function KomodoUnLoadDelayItems() {
    _gCommandmentSvc.finalize();
}

// All Komodo windows should be closed when the main Komodo window is closed.
// This is handled automatically for dialogs opened with the "dependent" or
// "dependent=yes" window feature, e.g.:
//      window.openDialog("chrome://.../foo.xul", "blah", "...dependent...");
// However some Komodo windows, e.g. help.xul, cannot use this window
// feature because it also implies that the window is always on top of the
// main Komodo window. Those windows are close by this unload handler.
function Komodo_CloseSubwindows() {
    log.info("Komodo_CloseSubwindows");
    try {
        var names = ["komodo_help"]; // Names of windows to close.
    
        var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                           .getService(Components.interfaces.nsIWindowMediator);
        for (var i = 0; i < names.length; ++i) {
            //XXX This is presuming that there is only ever zero or one
            //    instances of that window. If that is ever not true then
            //    use .getEnumerator() .getXULEnumerator() instead.
            var name = names[i]
            var w = wm.getMostRecentWindow(name);
            if (w) {
                log.info("Komodo_CloseSubwindows: close '"+name+"' window");
                w.close();
            }
        }
    } catch(ex) {
        log.error("Komodo_CloseSubwindows: "+ex);
    }
}

/*
    if you want to do something onUnload or onClose, then
    you have to register a handler function that takes no
    arguments.  This is called automaticaly.
    onOnload will call the handlers LIFO to reverse the
    order of initialization.
*/
var _gOnUnloadHandlers = [];
function KomodoOnUnload() {
    /* no further changes here, use registration functions below!

      we call onunload handlers LIFO
    */
    while (_gOnUnloadHandlers.length > 0) {
        var callback = _gOnUnloadHandlers.pop();
        try {
            if (callback.object) {
                callback.handler.apply(callback.object);
            } else {
                callback.handler();
            }
        } catch(e) {
            log.exception(e,"error when running '"+callback.handler+
                      "' shutdown handler (object='"+callback.object+"':");
        }
    }
    
    log.debug("made it to the end of komodoOnUnload\n");
}

function KomodoRegisterOnUnloadHandler(handler, object /*=null*/) {
    // Register a routine to be called on Komodo shutdown.
    // To register a simple routine do this:
    //      KomodoRegisterOnUnloadHandler(<routine>)
    // To register an object method do this:
    //      KomodoRegisterOnUnloadHandler(this.<method>, this);
    //XXX Do we want to add an optional argument list to pass in?
    if (typeof(object) == "undefined") object = null;
    var callback = new Object();
    callback.handler = handler;
    callback.object = object;
    _gOnUnloadHandlers.push(callback);
}


/**
 * KomodoCanClose
 *
 * this function checks with the projectmanager, and the viewmanagers
 * to see if it's ok to shutdown
 */
var _gCanCloseHandlers = [];
function KomodoCanClose()
{
    var retval;
    /* no further changes here, use registration functions below! */
    for (var i=_gCanCloseHandlers.length-1; i >= 0 ; i--) {
        var callback = _gCanCloseHandlers[i];
        try {
            if (callback.object) {
                retval = callback.handler.apply(callback.object);
            } else {
                retval = callback.handler();
            }
            if (!retval) return false;
        } catch(e) {
            log.exception(e,"error when running '"+callback.handler+
                        "' canclose handler (object='"+callback.object+"':");
        }
    }
    return true;
}

function KomodoRegisterCanCloseHandler(handler, object /* =null */) {
    // Register a routine to be called on Komodo shutdown.
    // and that can stop the quit behavior
    // To register a simple routine do this:
    //      KomodoRegisterCanCloseHandler(<routine>)
    // To register an object method do this:
    //      KomodoRegisterCanCloseHandler(this.<method>, this);
    //XXX Do we want to add an optional argument list to pass in?
    if (typeof(object) == "undefined") object = null;
    var callback = new Object();
    callback.handler = handler;
    callback.object = object;
    _gCanCloseHandlers.push(callback);
}

//XXX Mozilla's goQuitApplication() sends a "quit-application"
//XXX notification before tryToClose().  This notify causes nsPluginHostImpl
//XXX to unload all plugins, including Scintilla.  We call ProjectViewerCanClose
//XXX ourselves before passing control on to Moz.
var orig_goQuitApplication = window.goQuitApplication;

function KomodoGoQuitApplication() {
    if (!KomodoCanClose()) {
        return false;
    }
    return orig_goQuitApplication();
}

window.goQuitApplication = KomodoGoQuitApplication;

function NIY(e) {
    dump(e);
    alert("Not implemented yet");
}

















































































