/* Copyright (c) 2000-2001 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

// The Part wrapper class.  This wrapps koIParts into JS objects w/ a part attribute.
// This is done for two reasons: 1) performance -- accessing JS objects should be much faster
// than accessing XPCOM objects.  2) isolating run-time properties as opposed to serialized properties.
// Plugins will typically _read_ properties of the .part, but store runtime data in the wrapped object.

// gObserverSvc is located in statusbar.js :/

function partWrapper(part, depth) {
    var i;
    this.properties = {}; // filled in by extensions for view datapoints
    this.level = depth;
    this.part = part;
    this.type = 'file';
    this.iconstyle = {};
    this.iconopenstyle = {};
    this._name = null
    this.renameable = false;
    this._url = null;
    this.isContainerOpen = false;
    this.isContainer = false;
    this._children = null;
    this._depth = depth;
    this._file = null;

    // XXX getting the name should be in the plugins I guess
    if (part) {
        this.type = part.type;  // Performance optimization
        if (part.hasAttribute('name')) {
            this.properties['Name'] = this._name = part.getStringAttribute('name');
        }
        // This should be done by the file plugin.
        if (part.hasAttribute('url')) {
            this.url = part.getStringAttribute('url');
            if (!this._name) {
                this.properties['Name'] = this._name = uriparse_baseName(this.url);
            }
        }
        try {
            part = part.QueryInterface(Components.interfaces.koIContainerBase);
            // This should be moved out someday.
            this.isContainer = true;
            this.isContainerOpen = false;
        } catch (e) {/* no error condition here*/}
    }
    if (!this.name) this.name = this.type;

    /* pass the new part through the extension manager */
    extensionManager.callHandler(this, 'onUpdatePart', null);
    this.updateStatus();
    return this;    
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
partWrapper.prototype.constructor = partWrapper;

partWrapper.prototype.__defineGetter__("value",
function() 
{
    if (!this.part) return null;
    return this.part.value;
});

partWrapper.prototype.__defineSetter__("value",
function(value) 
{
    if (this.part) this.part.value = value;
});

partWrapper.prototype.__defineGetter__("name",
function() 
{
  return this._name;
});

partWrapper.prototype.__defineSetter__("name",
function(value) 
{
  this.properties['Name'] = this._name = value;
  if (this.part)
    this.part.setStringAttribute('name',this._name);
});

partWrapper.prototype.__defineGetter__("url",
function() 
{
  return this._url;
});

partWrapper.prototype.__defineSetter__("url",
function(value) 
{
    this._url = value;
    this._file = null;
    if (this.part)
        this.part.setStringAttribute('url',this._url);
});

partWrapper.prototype.__defineGetter__("file",
function() 
{
    // let the status service know we're ready for it
    if (!this._file && this._url && (this.type != 'folder' || this.isContainerOpen)) {
        var fsvc = Components.classes["@activestate.com/koFileService;1"] 
                         .getService(Components.interfaces.koIFileService)
        this._file = fsvc.getFileFromURI(this._url);
    }
    return this._file;
});

partWrapper.prototype.__defineSetter__("file",
function(value) 
{
    this._file = value;
});

partWrapper.prototype.__defineGetter__("children",
function() 
{
    if (this.isContainer && this._children == null) {
        //dump('getting children!\n');
        //dump(getStackTrace());
        this._children = [];
        var children = new Array();
        this.part.getChildren(children, new Object());
        if (children.value.length) {
            for (i=0; i< children.value.length; i++) {
                this._children[i] = new partWrapper(children.value[i], this._depth+1);
            }
        }
    }
    return this._children;
});

partWrapper.prototype.__defineSetter__("children",
function(value) 
{
  this._children = value;
});

partWrapper.prototype._updateKeyValue = function(key, data) {
    //this.properties = {};
    if (typeof(key) == 'undefined') return;
    if (typeof(data) == 'undefined') data = '0';
    //dump('key: '+key+' data: '+data+' url: '+this.url+'\n');
    switch(key) {
        case 'exists':
            if (!parseInt(data)) {
                this.properties['missing'] =
                    this.iconopenstyle['missing'] =
                    this.iconstyle['missing'] = 1;
            }
            break;
        case 'isReadOnly':
        case 'isReadWrite':
        case 'sccConflict':
        case 'sccNeedSync':
        case 'sccSync':
        case 'sccOk':
            this.properties[key] = 
                this.iconopenstyle[key] =
                this.iconstyle[key] = parseInt(data);
            break;
        case 'sccAction':
            //dump(this.url+" action "+data+"\n");
            this.iconopenstyle['edit'] = this.iconstyle['edit'] = 0;
            this.iconopenstyle['add'] = this.iconstyle['add'] = 0;
            this.iconopenstyle['delete'] = this.iconstyle['delete'] = 0;
            if (data) this.iconopenstyle[data] = this.iconstyle[data] = 1;
            this.properties[key] = data;
            break;
        case 'sccExclude':
            this.properties[key] = data=='1';
            break;
        default:
            this.properties[key] = data;
            break;
    }
    //dump('key: '+key+' data: ['+this.properties[key]+']\n');
}

partWrapper.prototype.updateStatus = function() {
    if (!this.file) return;
    try {
        var pKeys = {};
        var pValues = {};
        this.file.getStatusMap(pKeys, new Object(), pValues, new Object());
        // gProjectManager.log.debug('updating status for '+object.URI+' with '+pKeys.value.length+' items\n');
        for (var index = 0; index < pKeys.value.length; index++) {
            this._updateKeyValue(pKeys.value[index], pValues.value[index]);
        }
    } catch(e) {
        dump(e);
    }
}

partWrapper.prototype.hasChildren = function() {
    return this.isContainer && !this.part.isEmpty()
}

partWrapper.prototype.hasWrappedChildren = function() {
    return this.isContainer && this._children != null;
}

partWrapper.prototype.destroy = function() {
    if (this._children) {
        // Do the same thing to the children.
        for (i = 0; i < this._children.length; i++) {
            if (this._children[i])
                this._children[i].destroy();
            this._children[i] = null;
        }
        this._children = null;
    }
    this.part = null;
    this.file = null;
}

partWrapper.prototype.clone = function() {
    var c = new partWrapper(this.part, 0);
    var attr;
    for (attr in this) {
        c[attr] = this[attr];
    }
    if (this.part)
        c.part = this.part.clone();
    return c;
}

partWrapper.prototype.applyPartKeybindings = function() {
    if (!this.part) return;
    this.part.applyKeybindings(true)
}

/* this gets unique paths within a tree of parts.  If the part
   does not have children, its full path is returned.  In the case
   of a part having children, the directory for each child is added
   (there will be no dupes because a hash is used).
   This function is written primarily for use by scc. */
partWrapper.prototype.updateUniquePaths = function(urls,recursive /*=false*/) {
    if (typeof(recursive)=='undefined') recursive = false;
    if (recursive && this._children && this._children.length > 0) {
        if (this.url) {
            if (this.file.isDirectory) {
                urls[this.file.URI] = 1;
            } else {
                urls[this.file.dirName] = 1;
            }
        }
        for (var child in this._children) {
            this._children[child].updateUniquePaths(urls,recursive);
        }
    } else if (this.url && this.file) {
        urls[this.file.dirName] = 1;
    }
}

partWrapper.prototype.dump = function() {
    dump('--------------------------\n');
    dump('Type: '+this.type+'\n');
    dump('URL: '+this.url+'\n');
    dump('hasFile: '+(this.file != null)+'\n');
    if (this.file != null) {
        dump('FileURL: '+this.file.URI+'\n');
    }
    dump('hasChildren: '+this.hasChildren()+'\n');
    dump('hasWrappedChildren: '+this.hasWrappedChildren()+'\n');
    dump('Properties:\n');
    for (i in this.properties) {
        dump('    '+i+': '+this.properties[i]+'\n');
    }
    dump('IconStyle:\n');
    for (i in this.iconstyle) {
        dump('    '+i+': '+this.iconstyle[i]+'\n');
    }
    dump('IconOpenStyle:\n');
    for (i in this.iconopenstyle) {
        dump('    '+i+': '+this.iconopenstyle[i]+'\n');
    }
}

partWrapper.prototype.hasChild = function(child,recurse) {
    if (!this._children) return false;
    if (typeof(recurse)=='undefined') recurse = false;
    for (var i = 0; i < this._children.length; i++) {
        if (this._children[i] == child) 
            return true;
        if (recurse && this._children[i].hasChild(child,recurse))
            return true;
    }
    return false;
}

partWrapper.prototype.removeChild = function(child) {
    for (var i = 0; i < this._children.length; i++) {
        if (this._children[i] == child) {
            this._children.splice(i,1);
            this.part.removeChild(child.part);
            // let the status service know we're removing this
            if (child.url) {
                try {
                    gObserverSvc.notifyObservers(self, 'file_removed', child.url);
                } catch(e) { /* exception if no listeners */ }
            }
            child.destroy();
            return;
        }
    }
}

partWrapper.prototype.getDragData = function() {
    if (!this.part) return null;
    // if the part is not dragable, don't drag it
    if (this.part.hasAttribute('dragable') && !this.part.getBooleanAttribute('dragable')) return null;
    
    var flavors = new Object();
    this.part.getDragFlavors(flavors, new Object());
    var data = this.part.getDragData();
    var transferData = new TransferData();
    for (var i = 0; i < flavors.value.length; i++) {
        transferData.addDataForFlavour(flavors.value[i],data);
    }
    return transferData;
}

partWrapper.prototype.updateWrappedPart = function() {
    if (!this.isContainer ||
        (!this.isContainerOpen && !this._children)
        || !this.part) {
        return null;
    }
    // The 'part', wrapped in 'wrapped', has children or children of ... of children which may not be wrapped.
    var gotit;
    var child;
    var wrappedChild;
    /* get the part-type children */
    var children = new Array();
    this.part.getChildren(children, new Object());

    var unwrappedchildren = new Array();

    for (var i=0; i< children.value.length; i++) {
        child = children.value[i];
        /* we have a part, we don't know if it's wrapped or not */
        gotit = 0;
        /* iterate over the children of the wrapped part */
        for (var wrappedChildIndex in this._children) {
            wrappedChild = this._children[wrappedChildIndex];
            if (wrappedChild.part == child) {
                gotit = 1;
                /* check the grandchildren */
                try {
                    child = child.QueryInterface(Components.interfaces.koIContainerBase);
                } catch (e) {
                    child = null;
                }
                if (child) {
                    wrappedChild.updateWrappedPart();
                }
                /* pass the new part through the extension manager */
                extensionManager.callHandler(wrappedChild, 'onUpdatePart', null);                
                break;
            }
        }
        if (! gotit) {
            /* brand new part, needs wrapping */
            unwrappedchildren.push(child);
        }
    }

    /* firstchild is returned so that the tree view can scroll to an apropriate position */
    var firstchild = null;
    for (childIndex in unwrappedchildren ) {
        child = unwrappedchildren[childIndex];
        wrappedChild = new partWrapper(child, this.level+1);
        if (!firstchild) firstchild = wrappedChild;
        if (!this._children)
            this._children = [];
        this._children.push(wrappedChild);
    }
    if (firstchild) this.sortChildren();
    return firstchild;
}

partWrapper.prototype.sortChildren = function() {
    if (!this._children || this._children.length < 1) return;
    if (this._children.length > 1) {
        this._children.sort(compareWrappers);
    }
    for (var child in this._children) {
        this._children[child].sortChildren();
    }
}

var partwrapper_sortField = 'Name';
var partwrapper_sortDirection = 1;
function compareWrappers(a,b) {
    // this sorts first by the type of object, then by
    // the property specified by partwrapper_sortField
    
    // sort folders first
    if (a.type == 'folder' && b.type !='folder')
        return -1 * partwrapper_sortDirection;
    if (a.type != 'folder' && b.type =='folder')
        return 1 * partwrapper_sortDirection;
    // sort other types alphabeticaly
    if (a.type != b.type) {
        if (a.type < b.type)
            return -1 * partwrapper_sortDirection;
        else if (a.type > b.type)
            return 1 * partwrapper_sortDirection;
    }
    
    var au = typeof(a.properties[partwrapper_sortField]) == 'undefined';
    var bu = typeof(b.properties[partwrapper_sortField]) == 'undefined';
    // deal with non-existent columns
    if (au && bu) return 0;
    else if (au) return -1 * partwrapper_sortDirection;
    else if (bu) return 1 * partwrapper_sortDirection;
    var a1 = a.properties[partwrapper_sortField].toLowerCase();
    var b1 = b.properties[partwrapper_sortField].toLowerCase();
    if (Date.parse(a1)) {
        //dump('converting to date '+a1+' and '+b1+'\n');
        a1 = Date.parse(a1);
        b1 = Date.parse(b1);
    } else if (parseFloat(a1)) {
        //dump('converting to number '+a1+' and '+b1+'\n');
        a1 = parseFloat(a1);
        b1 = parseFloat(b1);
    }
    //dump('comparing '+partwrapper_sortField+' '+a1+' and '+b1+'\n');
    // ok, they both have the property, compare
    if (a1 < b1)
        return -1 * partwrapper_sortDirection;
    else if (a1 > b1)
        return 1 * partwrapper_sortDirection;
    return 0;
}
