/* Copyright (c) 2000-2001 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

// start the extension
registeredExtensions[registeredExtensions.length] = new peFolder();

function peFolder() {
    this.name = 'peFolder';
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
peFolder.prototype.constructor = peFolder;

peFolder.prototype.init = function() {
}

peFolder.prototype.registerCommands = function() {
    extensionManager.registerCommand('cmd_delete', this);
    extensionManager.registerCommand('cmd_importFromFS', this);
    extensionManager.registerCommand('cmd_importFromFS_Project', this);
}

peFolder.prototype.registerMenus = function() {
    extensionManager.createMenuItem(Components.interfaces.koIPart_folder,'Import from File System...','cmd_importFromFS');
    extensionManager.createMenuItem(Components.interfaces.koIPart_folder,'Refresh Status','cmd_refreshStatus');
    var menupopup = document.getElementById('folder_context');
    extensionManager.addMenuItem(Components.interfaces.koIPart_folder, menupopup);
}

peFolder.prototype.registerEventHandlers = function() {
    extensionManager.addEventHandler(Components.interfaces.koIPart,'onUpdatePart',this);
}

peFolder.prototype.supportsCommand = function(command, item) {
    //dump('for ' + command + '\n');
    if (!gFocusedProjectView) return false;
    var items = gFocusedProjectView.getSelectedItems();
    var havemultiple = items.length > 1;
    //dump('for ' + command + ', items.length == ' + items.length + '\n');
    switch (command) {
    case 'cmd_delete':
        return true;
    case "cmd_importFromFS_Project":
        return gProjectManager.getCurrentProject() != null;
    case "cmd_importFromFS":
        var ok =false;
        if (!item)
            item = gFocusedProjectView.getSelectedItem();
        if (item) {
            try {
                ok = item.part.QueryInterface(Components.interfaces.koIProject) != null;
            } catch (e) {}
            if (!ok) {
                try {
                    ok = item.part.QueryInterface(Components.interfaces.koIPart_folder) != null;
                } catch (e) {}
            }
        }
        return ok;
    default:
        break;
    }
    return false;
}

peFolder.prototype.isCommandEnabled = function(command, item) {
    //dump('for ' + command + '\n');
    var items = null;
    
    if (gFocusedProjectView) {
        items = gFocusedProjectView.getSelectedItems();
        var havemultiple = items.length > 1;
    }
    switch (command) {
        case 'cmd_delete':
            // Delete is enabled for editor views and for any part
            // selection that isn't a project.
            if (view_editorHasFocus()) {
                //dump('editor has focus\n');
                return gViewMgr.currentView.getAttribute('type') == 'editor';
            }
            // Make sure that the focus is in the project manager or the toolbox
            if (!view_projectHasFocus() && !view_toolboxHasFocus()) {
                return false;
            }
            if (!item)
                item = gFocusedProjectView.getSelectedItem();
            //dump('we got the selected item\n');
            //item.dump();
            if (!item) {
                //dump('nothing to delete\n');
                return false; // nothing to delete
            }
            try {
                var i = item.part.QueryInterface(Components.interfaces.koIProject);
            } catch (e) {
                // if the interface is not supported, it is not a project, so return true.
                //dump("not a project, saying yes we can delete\n");
                return true;
            }
            return false; // we don't allow delete on projects.
        case "cmd_importFromFS_Project":
            return gProjectManager.getCurrentProject() != null;
        case "cmd_importFromFS":
            var ok =false;
            if (!item)
                item = gFocusedProjectView.getSelectedItem();
            if (item) {
                try {
                    ok = item.part.QueryInterface(Components.interfaces.koIProject) != null;
                } catch (e) {}
                if (!ok) {
                    try {
                        ok = item.part.QueryInterface(Components.interfaces.koIPart_folder) != null;
                    } catch (e) {}
                }
            }
            return ok;
        default:
            break;
    }
    return false;
}

peFolder.prototype.doCommand = function(command) {
    var item = null;
    switch (command) {
    case 'cmd_addFilePart_Project':
        gProjectManager.viewMgr.focus();
        item = gProjectManager.getCurrentProject();
        // fall through
    case 'cmd_addFilePart':
        if (!item) item = gFocusedProjectView.getSelectedItem();
        this.addFile(item);
        break;
    case 'cmd_addNewFile':
        if (!item) item = gFocusedProjectView.getSelectedItem();
        this.addNewFile(item);
        break;
    case 'cmd_addRemoteFilePart_Project':
        gProjectManager.viewMgr.focus();
        item = gProjectManager.getCurrentProject();
        // fall through
    case 'cmd_addRemoteFilePart':
        if (!item) item = gFocusedProjectView.getSelectedItem();
        peFolder_addRemoteFile(item);
        break;
    case 'cmd_addFolderPart_Project':
        gProjectManager.viewMgr.focus();
        item = gProjectManager.getCurrentProject();
        // fall through
    case 'cmd_addFolderPart':
        if (!item) item = gFocusedProjectView.getSelectedItem();
        var fitem = new Object();
        fitem.name = 'New Folder';
        fitem.value = null;
        if (gFocusedProjectView.manager.renameItem(fitem)) {
            this.addFolder(fitem.name, item);
        }
        break;
    case 'cmd_delete':
        var items = gFocusedProjectView.getSelectedItems();
        var havemultiple = (items.length > 1);
        if (havemultiple) {
            var question = "Do you want to remove the " + items.length +
                           " items you have selected?";
            if (dialog_yesNo(question) != "Yes")
                return;
        }
        for (var i = 0; i < items.length; i++) {
            gFocusedProjectView.manager.removeItem(items[i], havemultiple);
        }        
        break;
    case "cmd_importFromFS_Project":
        gProjectManager.viewMgr.focus();
        item = gProjectManager.getCurrentProject();
    case "cmd_importFromFS":
        if (!item) item = gFocusedProjectView.getSelectedItem();
        if (!item) return;
        this.ImportFromFS(item);
        break;
    default:
        break;
    }    
}

peFolder.prototype.addNewFile = function(item)
{
    // Get base directory from item.  It can be a folder or a project.
    var defaultDirectory = null;
    var url;
    if (item) {
        url = item.part.getStringAttribute('url')
        var pathSvc = Components.classes["@activestate.com/koOsPath;1"]
                                .getService(Components.interfaces.koIOsPath);
        defaultDirectory = uriparse_URIToLocalPath(url);
        if (item.part.type == 'project') {
            defaultDirectory = pathSvc.dirname(defaultDirectory);
        }
    } 
    var newfilename = filepicker_saveFile(defaultDirectory);
    if (!newfilename) return;
    // We need to create an empty file, add it to the 'parentitem' and open it.
    var sysUtils = Components.classes["@activestate.com/koSysUtils;1"]
                            .createInstance(Components.interfaces.koISysUtils);
    sysUtils.Touch(newfilename);
    url = uriparse_localPathToURI(newfilename);
    peFolder_addFileWithURL(url, item);
    open_openURI(newfilename);
}

function peFolder_addFileWithURL(url, item)
{
    var index = -1;
    if (typeof(item)=='undefined' || !item)
        item = gFocusedProjectView.getSelectedItem();
    if (item) index = gFocusedProjectView.getIndexByPart(item);    
    var part;
// #if PRODUCT_TYPE == "professional"
    if (url.match(/\.ui$/i)) {
        part = Components.classes["@activestate.com/koPart?type=dialog;1"]
                                    .createInstance(Components.interfaces.koIPart_dialog);
    } else {
// #endif
        part = Components.classes["@activestate.com/koPart?type=file;1"]
                                    .createInstance(Components.interfaces.koIPart_file);
// #if PRODUCT_TYPE == "professional"
    }
// #endif
    part.setStringAttribute('url', url);
    part.setStringAttribute('name', uriparse_baseName(url));
    gFocusedProjectView.manager.addItem(part,index);
}

peFolder.prototype.onUpdatePart = function(item,event)
{
    // event is null for onUpdatePart
    var isProject = null;
    try {
        isProject = item.part.QueryInterface(Components.interfaces.koIProject);
    } catch (e) {    }
    
    item.renameable = !item.part.hasAttribute('url');
    
    if (item.isContainer && !isProject) {
        if (!item.url) {
            item.properties['tooltiptext'] = '';
            item.iconopenstyle['projectVirtualFolderOpen'] = 1;
            item.iconstyle['projectVirtualFolderClosed'] = 1;
        } else {
            item.properties['tooltiptext'] = uriparse_displayPath(item.url);
            item.iconopenstyle['projectFolderOpen'] = 1;
            item.iconstyle['projectFolderClosed'] = 1;
        }
    }
}

function peFolder_add(type, menuType)
{
    try {
        var target, item, items;
        var tabs, tab;
        tabs = document.getElementById('project_toolbox_tabs');
        // menuType can be:
        //   0: from a context menu on a selected part
        //   1: from the toplevel toolbox menuor the context menu on the toolbox tab
        //   2: from the toplevel project menu
        switch (menuType) {
            case 0:
                target = gFocusedProjectView.getSelectedItem();
                break;
            case 1:
                target = toolboxMgr.viewMgr.view;
                uilayout_ensureToolboxPaneShown();
                gFocusedProjectView = target;
                break;
            case 2:
                uilayout_ensureProjectPaneShown();
                target = gProjectManager.getCurrentProject();
                gFocusedProjectView = gProjectManager.viewMgr.view;
                break;
            default:
                log.error("Unknown menuType in peFolder_add: " + menuType);
        }
        // type can be:
        //   'files': add existing files
        var name;
        switch (type) {
            case 'files':
                peFolder_addFile(target);
                break;
            case 'snippet':
                peSnippet_addSnippet(target);
                break;
            case 'command':
                peCommand_addCommand(target);
                break;
            case 'dialog':
                peGUI_addDialog(target);
                break;
// #if WITH_CUSTOM_MENUS_TOOLBARS
            case 'menu':
                peMenu_addMenu(target);
                break;
            case 'toolbar':
                peMenu_addToolbar(target);
                break;
// #endif
            case 'URL':
                peURL_addURL(target);
                break;
            case 'macro':
                peMacro_addMacro(target);
                break;
            case 'webservice':
                peWebService_addService(target);
                break;
            case 'remotefiles':
                peFolder_addRemoteFile(target);
                break;
            case 'template':
                peTemplate_addTemplate(target);
                break;
            case 'directoryshortcut':
                dirname = filepicker_getFolder();
                if (!dirname) return;
                peFile_addDirectoryShortcut(target, dirname);
                break;
            case 'newfolder':
                name = dialog_prompt("Enter folder name:");
                if (!name) return;
                peFolder_addFolder(name, target);
                break;
            default:
                log.error("Unknown item type called to peFolder_add: " + type);
        }
    } catch (e) {
        log.error(e);
    }
}

function peFolder_addFile(parent_item)
{
    //XXX todo: Support other resources 

    var defaultDir = null;
    if (parent_item && parent_item.url) {
        defaultDir = uriparse_dirName(parent_item.url)
    }
    var files = filepicker_openFiles(defaultDir, // default dir
                                     null, // default filename
                                     "Add Files to Project"); // title
    if (files == null) {
        return false;
    } else {
        var url;
        for (var i = 0; i < files.length; ++i) {
            url = uriparse_localPathToURI(files[i]);
            peFolder_addFileWithURL(url, parent_item);
        }
        return true;
    }
}

function peFolder_addRemoteFile(item)
{
    var list = filepicker_browseForRemoteFiles(); // in fileops.js
    if (list) {
        for (var i = 0; i < list.length; i++) {
            peFolder_addFileWithURL(list[i], item);
        }
        return true;
    }
    return false;
}

function peFolder_addFolder(name, item)
{
    var index = gFocusedProjectView.getIndexByPart(item);
    var folder = Components.classes["@activestate.com/koPart?type=folder;1"]
                                    .createInstance(Components.interfaces.koIPart_folder);
    folder.name = name;
    gFocusedProjectView.manager.addItem(folder,index);
}

peFolder.prototype.ImportFromFS = function(item)
{
    if (typeof(item) == 'undefined' || !item) 
        item = gFocusedProjectView.getSelectedItem();
    if (!item) return false;

    var resp = new Object();
    resp.include = item.part.prefset.getStringPref("import_include_matches");
    resp.exclude = item.part.prefset.getStringPref("import_exclude_matches");
    resp.recursive= item.part.prefset.getBooleanPref("import_recursive");
    resp.importType= item.part.prefset.getStringPref("import_type");
    if (item.part.prefset.hasPrefHere("import_dirname")) {
        resp.dirname = item.part.prefset.getStringPref("import_dirname");
    } else {
        if (item.part.hasAttribute('url')) {
            // Is it a project or a folder?
            var url = item.part.getStringAttribute('url')
            var pathSvc = Components.classes["@activestate.com/koOsPath;1"]
                                    .getService(Components.interfaces.koIOsPath);
            resp.dirname = uriparse_URIToLocalPath(url);
            if (item.part.type == 'project') {
                resp.dirname = pathSvc.dirname(resp.dirname );
            }
        } else {
            var os = Components.classes["@activestate.com/koOs;1"].getService();
            resp.dirname = os.getcwd();
        }
    }
    resp.res = '';
    resp.item = item;
    resp.global = window;
    
    openWindowUniqueInstance(
        "chrome://jaguar/content/project/importFromFS.xul",
        'komodo_import',
        "chrome,dialog,modal",
        resp);
    if (resp.res != true) {
        return false;
    }

    if (resp.include != item.part.prefset.getStringPref("import_include_matches")) {
        item.part.prefset.setStringPref("import_include_matches", resp.include);
    }
    if (resp.exclude != item.part.prefset.getStringPref("import_exclude_matches")) {
        item.part.prefset.setStringPref("import_exclude_matches", resp.exclude);
    }
    if (resp.recursive != item.part.prefset.getBooleanPref("import_recursive")) {
        item.part.prefset.setBooleanPref("import_recursive", resp.recursive);
    }
    if (resp.importType != item.part.prefset.getStringPref("import_type")) {
        item.part.prefset.setStringPref("import_type", resp.importType);
    }
    if (resp.dirname != item.part.prefset.getStringPref("import_dirname")) {
        item.part.prefset.setStringPref("import_dirname", resp.dirname);
    }

    return true;
}

