/* Copyright (c) 2000-2001 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

// start the extension
var peWSMgr = new peWebService();
registeredExtensions[registeredExtensions.length] = peWSMgr;

function peWebService() {
    this.name = 'peWebService';
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
peWebService.prototype.constructor = peWebService;

peWebService.prototype.init = function() {
    this.log = getLoggingMgr().getLogger("peWebService");
    this.registerCommands();
}

peWebService.prototype.registerCommands = function() {
    try {
        extensionManager.registerCommand('cmd_viewWebService', this);
        extensionManager.registerCommand('cmd_insertWebService', this);
        extensionManager.registerCommand('cmd_generateProxyWebService', this);
        extensionManager.registerCommand('cmd_helpWebService', this);
    } catch (e) {
        this.log.error(e);
    }
}
peWebService.prototype.registerEventHandlers = function() {
    try {
        extensionManager.addEventHandler(Components.interfaces.koIPart_webservice,'onUpdatePart',this);
        extensionManager.addEventHandler(Components.interfaces.koIPart_webservice,'ondblclick',this);
    } catch (e) {
        this.log.error(e);
    }
}
peWebService.prototype.registerMenus = function() {
    try {
        extensionManager.createMenuItem(Components.interfaces.koIPart_webservice,
                                        'View WSDL...','cmd_viewWebService');
        extensionManager.createMenuItem(Components.interfaces.koIPart_webservice,
                                        'Insert Service Call...','cmd_insertWebService');
        extensionManager.createMenuItem(Components.interfaces.koIPart_webservice,
                                        'Generate Proxy...','cmd_generateProxyWebService');
        extensionManager.createMenuItem(Components.interfaces.koIPart_webservice,
                                        'Service Help...','cmd_helpWebService');
    } catch (e) {
        this.log.error(e);
    }
}

peWebService.prototype.supportsCommand = function(command, item) {
    switch (command) {
    case 'cmd_insertWebService':
        var view = gViewMgr.currentView;
        if (!view) return false;
        if (!WSSetup_SupportedLanguage(view.document.language)) return false;
    case 'cmd_helpWebService':
    case 'cmd_generateProxyWebService':
    case 'cmd_viewWebService':
        if (!item) {
            item = gFocusedProjectView.getSelectedItem();
        }
        return item && item.part.hasAttribute('wsdl');
    default:
        break;
    }
    return false;
}

peWebService.prototype.isCommandEnabled = peWebService.prototype.supportsCommand;

peWebService.prototype.doCommand = function(command) {
    var item;
    switch (command) {
    case 'cmd_helpWebService':
        item = gFocusedProjectView.getSelectedItem();
        if (item.part.hasAttribute('wsdl')) {
            this.doHelp(item);
        }
        break;
    case 'cmd_insertWebService':
        item = gFocusedProjectView.getSelectedItem();
        if (item.part.hasAttribute('wsdl')) {
            insertSimpleWebService(item);
        }
        break;
    case 'cmd_generateProxyWebService':
        item = gFocusedProjectView.getSelectedItem();
        if (item.part.hasAttribute('wsdl')) {
            generateWebServiceProxy(item.part.getStringAttribute('name'),item.part.getStringAttribute('wsdl'));
        }
        break;
    case 'cmd_viewWebService':
        item = gFocusedProjectView.getSelectedItem();
        if (item.part.hasAttribute('wsdl')) {
            gViewMgr.doFileOpen(item.part.getStringAttribute('wsdl'));
        }
        break;
    default:
        break;
    }
}

peWebService.prototype.ondblclick = function(item,event) {
    if (item.part.hasAttribute('wsdl')) {
        insertSimpleWebService(item);
    }
}

peWebService.prototype.onUpdatePart = function(item,event)
{
    item.iconopenstyle['WebService'] = 1
    item.iconstyle['WebService'] = 1
    item.renameable = 0;
}

peWebService.prototype.doHelp = function(item)
{
    var os = Components.classes["@activestate.com/koOs;1"].getService();
    var bmMgr = Components.classes["@activestate.com/koBookmarkManager;1"]
                    .getService(Components.interfaces.koIBookmarkManager);
    var path = os.path.join(bmMgr.rootDirectory, item.name + ".html");
    gViewMgr.doFileOpen(path,'browser');
}


function peWebService_addService(item)  {
    var obj = new Object();
    obj.type = 'add';
    obj.dir = null;
    obj.url = null;
    obj.retval = false;
    window.openDialog("chrome://jaguar/content/webservices/bmedit.xul", 
               "komodo_wsbmedit",
               "chrome,close=yes,modal", obj);
    
    // now get the info from obj, and add the webservice
    if (obj.retval && obj.bmark) {
        insertWebServicePart(obj.bmark, obj.url, item);
    }
}

var peWebService_editProperties = function(item)  {
    var wsdl = item.part.getStringAttribute('wsdl');
    var obj = new Object();
    obj.type = 'edit';
    obj.name = item.name;
    obj.url = wsdl;
    obj.description = item.value;
    obj.shortname = item.part.getStringAttribute('object');
    obj.retval = false;
    window.openDialog("chrome://jaguar/content/webservices/bmedit.xul", 
               "komodo_wsbmedit",
               "chrome,close=yes,modal", obj);
    
    if (obj.retval)  { // ok was clicked
        item.name = obj.name;
        item.part.setStringAttribute('wsdl',obj.url);
        item.part.setStringAttribute('object', obj.shortname);
        item.part.value = obj.description;
    }
}

function _generateWebServiceDetails(url) {
    // get details from the wsdl if possible
    window.setCursor("spinning");
    try {
        var bmark = Components.classes["@activestate.com/koBookmark;1"].createInstance(Components.interfaces.koIBookmark);
        bmark.url = url;
        var errmsg = new Object();
        var success = bmark.generateFromWSDL(errmsg);
        if (!success) {
            window.setCursor("auto");
            dialog_alert("Error retrieving " + bmark.url + ".  " + errmsg.value);
            return null;
        }
        success = bmark.save(bmark.name, false, errmsg);
        if (errmsg.value) {
            window.setCursor("auto");
            dialog_alert(errmsg.value);
            return null;
        }
    } catch(e) {
        log.exception(e);
    }
    window.setCursor("auto");
    return bmark;
}

function insertWebServicePart(bmark, url, item, index)
{
    if (typeof('index')=='undefined') index = -1;
    window.setCursor("spinning");
    var folderpart = Components.classes["@activestate.com/koPart?type=webservice;1"]
                                    .createInstance(Components.interfaces.koIPart_webservice);
    folderpart.setStringAttribute('wsdl', url);
    folderpart.setStringAttribute('name', bmark.name);
    folderpart.setStringAttribute('object', bmark.shortname);
    folderpart.value = bmark.description;
    if (typeof(item)=='undefined' || !item)
        item = gFocusedProjectView.getSelectedItem();
    if (item) index = gFocusedProjectView.getIndexByPart(item);    
    gFocusedProjectView.manager.addItem(folderpart,index);
    window.setCursor("auto");
}

function AddWebServiceAtIndex(url,index)
{
    // XXX for now use the koIBookmark to parse the wsdl
    var bmark = _generateWebServiceDetails(url);
    if (!bmark) return;
    insertWebServicePart(bmark, url, null, index);
    //dump("leaving AddURL\n");
}

function AddWebService(url)
{
    AddWebServiceAtIndex(url,-1)
}

function addWSBookmarkToToolbox(bmark, url)
{
    gFocusedProjectView = toolboxMgr.viewMgr.view;
    gFocusedProjectView.manager.viewMgr.focus();
    insertWebServicePart(bmark, url);
}

function generateWebServiceProxy(name, wsdl, language) {
    // XXX deal with language selection
    if (typeof(language) == 'undefined') {
        // XXX this is where we ask what language to generate the proxy for
        language = 'Perl';
    }
    var os = Components.classes["@activestate.com/koOs;1"].getService();
    var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"]
               .getService(Components.interfaces.koILastErrorService)

    var useCustomDir = gPrefs.hasLongPref('wsPreferCustomDirectory')?gPrefs.getLongPref('wsPreferCustomDirectory'):0;
    var addtoProj = gPrefs.hasBooleanPref('wsAddToCurrentProject')?gPrefs.getBooleanPref('wsAddToCurrentProject'):true;
    var directory = gPrefs.hasStringPref(language+'WebServiceDirectory')?gPrefs.getStringPref(language+'WebServiceDirectory'):null;
    var project = gProjectManager.getCurrentProject();
    if (!useCustomDir && project) {
        directory = uriparse_dirName(project.url);
    }
    
    //XXX I am pretty sure that this block of code is broken, given changes in
    //    the sematics of the various URI <--> local filename conversion
    //    routines. This code is too convoluted to bother with right now. If
    //    someone cares to explain the intent I will be happy to rewrite it.
    //    --TM.
    var leafname = name+'.pm';
    var url = leafname;
    var localPath = null;
    if (directory) {
        var local = false;
        url=directory+'/'+leafname;
        try {
            var localfile = uriparse_URIToLocalPath(url);
            if (localfile) localPath = localfile.path;
        } catch(e) {
            // not a file url
            local = true;
        }
        if (local) {
            try {
                localPath = os.path.join(directory,leafname);
                url = uriparse_localPathToURI(localPath);
            } catch(e) {
                // file does not exist
            }
        }
    }
    var part = null;
    if (project) {
        part = project.part.getChildWithTypeAndStringAttribute('file', 'url', url, true);
    }
    // see if the current project already has the proxy, if so, ask if they want to
    // replace the file in the current project.
    var generate = true;
    var question;
    if (project && addtoProj && part) {
        if (!localPath) localPath = url;
        question = "The proxy ["+localPath+"] already exists in the "+
                       "current project, would you like to refresh the proxy?";
        if (dialog_yesNo(question) != "Yes")
            generate = false;
    } else if (localPath && os.path.exists(localPath)) {
        // see if the url already exists, if so, what do we do?
        question = "The proxy ["+localPath+"] already exists in the "+
                       "current project, would you like to refresh the proxy?";
        if (dialog_yesNo(question) != "Yes")
            generate = false;
    }
    
    window.setCursor("spinning");
    if (generate) {
        // generate the proxy now
        var proxygen = Components.classes["@activestate.com/KoWSProxyGenerator;1"].
                    getService(Components.interfaces.koIWSProxyGenerator);
        var text;
        try {
            text = proxygen.generate(language, wsdl);
        } catch(e) {
            window.setCursor("auto");
            var error = lastErrorSvc.getLastErrorMessage();
            if (error)
                dialog_alert(error);
            else
                dialog_alert("Unknown error parsing WSDL document");
            return;
        }
        
        // is the file already open?
        var view = gViewMgr.getViewForFile(url);
        if (view) {
            //select the view
            view.makeCurrent();
        } else {
            view = gViewMgr.doNewView(language);
            // a little hack to update the name
            view.document.baseName = leafname;
            view.parentView.updateLeafName(view);
        }
        view.document.buffer = text;
        window.setCursor("auto");
        if (directory) {
            view.saveAsURI(url);
        } else {
            view.saveAs();
        }
    } else { // not generating
        gViewMgr.doFileOpen(url);
    }
    
    // add it to the project
    if (addtoProj && project && !part) {
        var filepart = Components.classes["@activestate.com/koPart?type=file;1"]
                                        .createInstance(Components.interfaces.koIPart_file);
        filepart.setStringAttribute('url', url);
        gProjectManager.addItem(filepart);
    }
    window.setCursor("auto");
}

/******************************************************/
// old web services js functions


// XXX insertWebService() and getWSInserter() are near-duplicates of the
// functions of the same name in bookmarks.js.  These should be factored out
// into a common file somewhere.
function insertSimpleWebService(item) {
    if (typeof(item) == 'undefined' || !item) {
        item = gFocusedProjectView.getSelectedItem();
        if (!item || !item.part.hasAttribute('wsdl')) return false;
    }
        
    // First, run the WS setup wizard for this language if it has not been
    // run before.
    var view = gViewMgr.currentView;
    if (!view) return 0;
    var language = view.document.language;
    try {
        WSSetup_SetupIfHaveNotAlready(language)
    } catch(e) {
        // an exception is thrown if it is an unsupported language, ignore and escape
        log.exception(e);
        return false;
    }

    var shortname = item.part.getStringAttribute('object');
    var url = item.part.getStringAttribute('wsdl');
    // Do the WS insertion magic.
    var wsinserter = view.document.languageObj.
        getLanguageService(Components.interfaces.koIWSInsertLanguageService);
    if (wsinserter) {
        // XXX do away with scimoz usage here
	wsinserter.insert(view.scimoz, 
			  view.scimoz.currentPos, 
			  shortname, 
			  url);
        gViewMgr.currentView.setFocus();
    }
    return true;
}

function getWSInserter() {
    var v = gViewMgr.currentView;
    scin = v._scimoz;
    var wsinserter = v.py_inst.sourceFile.getLanguageBlockAtPosition(0).language.
        getLanguageService(Components.interfaces.koIWSInsertLanguageService);
    return wsinserter;
}    

