/* Copyright (c) 2000-2003 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

/* Komodo view handling.
 * This is a singleton which provides simple interface to
 *   - currentView: The current view
 *
 * It also holds the controller for toplevel view-related commands
 * and "command-like" code to open files for example.

 * It manages internally which is the toplevel viewlist
 * (to which new views are added by default)

INTERFACE:

    gViewMgr is defined globally.
    
    gViewMgr.currentView: The currently focused view

    gViewMgr.topView: The toplevel viewlist -- this is what new views
                      get added to by default.

    view_openFilesWithPicker(...)
        Asks uses for which files to open, and opens them
        (as projects, GUI files or standard files).

The viewMgr is also a controller for "top-level" commands such as
"cmd_bufferClose".

The viewMgr is an observer on "view_opened" and "view_closed" and
"current_view_changed" notifications.

OPTIMIZATION NOTE:

The viewMgr could also listen for "select" events, to filter out 'select' events that
don't correspond to real selection changes.  At this point that is not done.

*/

var gViewMgr = null;

function views_onload() {
    gViewMgr = new viewManager();
    var viewSvc = Components.classes['@activestate.com/koViewService;1'].getService(
                    Components.interfaces.koIViewService);
    viewSvc.setViewMgr(gViewMgr);
};

function viewManager() {
    this.log = getLoggingMgr().getLogger('views');
    this.log.info("viewManager constructor");
    this.currentView = null;
    this.topView = document.getElementById('topview');
    this.topView.init();
    KomodoRegisterCanCloseHandler(this.offerToSave, this);
    KomodoRegisterOnUnloadHandler(this.shutdown, this);
    this.docSvc = Components.classes['@activestate.com/koDocumentService;1']
                    .getService(Components.interfaces.koIDocumentService);
    this.observerSvc = Components.classes["@mozilla.org/observer-service;1"].
                    getService(Components.interfaces.nsIObserverService);
    this.lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
                            getService(Components.interfaces.koILastErrorService);
    this.observerSvc.addObserver(this, "open_file", false); // commandment
    this.observerSvc.addObserver(this, "SciMoz:FileDrop", false); 
    this.observerSvc.addObserver(this, "view_opened", false);
    this.observerSvc.addObserver(this, "view_closed", false);
    this.observerSvc.addObserver(this, "current_view_changed", false);
    this.observerSvc.addObserver(this, "file_status", false);



    this._viewCount = 0;
    this.batchMode = false;
    this.lastviewcache = this.cacheCommandData(null);
    window.controllers.appendController(this); // XXX need to uninstall on quit
};

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
viewManager.prototype = new Controller();
viewManager.prototype.constructor = viewManager;

viewManager.prototype.shutdown = function()
{
    try {
        this.observerSvc.removeObserver(this, "open_file");  // commandment
        this.observerSvc.removeObserver(this, "SciMoz:FileDrop"); 
        this.observerSvc.removeObserver(this, "view_opened");
        this.observerSvc.removeObserver(this, "view_closed");
        this.observerSvc.removeObserver(this, "current_view_changed");
        this.observerSvc.removeObserver(this, "file_status");



    } catch(e) {
        /* moz probably already removed them */
        log.warn('possible error shutting down viewManager:'+e);
    }
    this.docSvc = null;
    this.observerSvc = null;
    gViewMgr = null;
}


viewManager.prototype.newTemplate = function() {
    try {
        this.log.info("doing newTemplate: ");
        var uri;
        // Get template selection from the user.
        var obj = new Object();
        window.openDialog("chrome://jaguar/content/templates/new.xul",
                          "_blank",
                          "chrome,modal,titlebar",
                          obj);
        if (obj.template == null) return;
    
        if (obj.addToToolbox) {
            try {
                uri = uriparse_localPathToURI(obj.template);
                AddTemplateToToolbox(uri);
                uilayout_ensureToolboxPaneShown(window);
            } catch (ex) {
                this.log.exception(ex, "Error adding template to toolbox.");
            }
        }
    
        uri = uriparse_localPathToURI(obj.template);
        this.doFileNewFromTemplate(uri);
    } catch(ex) {
        this.log.exception(ex, "Error in newTemplate.");
    }
}


viewManager.prototype.doFileNewFromTemplate = function(uri,
                                                       viewType /*="editor"*/,
                                                       viewList /*=null*/)
{
    this.log.info("doing doFileNewFromTemplate: ");
    if (typeof(viewType) == "undefined" || viewType == null) viewType = "editor";
    if (typeof(viewList) == "undefined") viewList = null;
    
    // Read the template.
    var lastErrorSvc = Components.classes['@activestate.com/koLastErrorService;1'].getService();
    var localPath = uriparse_URIToLocalPath(uri);
    var basename = uriparse_baseName(uri);
    var ext = uriparse_ext(uri);
    var name = basename.slice(0, basename.length-ext.length);

    var osSvc = Components.classes['@activestate.com/koOs;1'].getService();
    var content;
    try {
        //XXX .readfile() should use last error service and should use it here.
        content = osSvc.readfile(localPath);
    } catch(ex) {
        dialog_alert("Could not read template file contents: '"+uri+"'");
        return;
    }

    // Warn about backward in-compatibility in template codes.
    pattern = /\[\s*komodo-variable: \$(\w+)\s*\]/;
    if (content.match(pattern)) {
        dialog_alert("It looks like this template is using Komodo's old "+
            "system of special template codes.  This mechanism is "+
            "deprecated and will be removed in the next version of Komodo.  "+
            "You should upgrade your templates to use the new, more "+
            "capable, mechanism (see 'Interpolation Shortcuts' in Help).",
            null, "Deprecated Template Codes", "old_template_codes");
        content = this.docSvc.deprecatedInterpolateTemplate(content);
    }

    // Interpolate any codes.
    var icontent = null;
    try {
        var istrings = Interpolate_interpolate(
                          window,
                          [content],
                          true, // bool: codes are bracketed
                          "Template '"+name+"' Query");
        icontent = istrings[0];
    } catch (ex) {
        var errno = lastErrorSvc.getLastErrorCode();
        if (errno == Components.results.NS_ERROR_ABORT) {
            // Command was cancelled.
        } else if (errno == Components.results.NS_ERROR_INVALID_ARG) {
            var errmsg = lastErrorSvc.getLastErrorMessage();
            dialog_alert("Could not interpolate:" + errmsg);
        } else {
            log.exception(ex, "Error interpolating template.");
            dialog_internalError("Could not process interpolation codes "+
                                 "in template '"+basename+"'.",
                                 "Error interpolating template '"+
                                 uri+"'.\n\n"+gTimeline.getStack());
        }
        return;
    }

    // Load the template.
    MRU_addURL("mruTemplateList", uri);
    var doc = this.docSvc.createDocumentFromTemplate(icontent, name, ext);
    if (viewList) {
        view = viewList.createViewFromDocument(doc, viewType);
    } else {
        view = this.topView.createViewFromDocument(doc, viewType);
    }
}


viewManager.prototype.doNewView = function(language /*= prefs.fileDefaultNew*/,
                                           viewType /*='editor'*/) {
    this.log.info("doing doNewView: ");
    gTimeline.enter('viewManager.doNewView');

    if (typeof(viewType)=='undefined' || !viewType)
        viewType = 'editor';
    if (typeof(language)=='undefined' || !language) {
        language = gPrefs.getStringPref('fileDefaultNew');
    }

    // the following line is delayed to avoid notifications during load()
    var doc = this.docSvc.createUntitledDocument(language);
    var view = this.topView.createViewFromDocument(doc,viewType);
    
    gTimeline.leave('viewManager.doNewView');
    this.log.info("leaving doNewView");
    return view;
}

viewManager.prototype.newViewFromURI = function(uri, viewType/*='editor'*/, viewList/*=null*/)
{
    this.log.info("doing newViewFromURI: " + uri);
    gTimeline.enter('viewManager.newViewFromURI');
    if (typeof(viewType)=='undefined' || !viewType)
        viewType = 'editor';
    if (typeof(viewList)=='undefined')
        viewList = null;

    var doc = this.docSvc.createDocumentFromURI(uri);
    var view = null;
    if (! doc.file.exists) {
        if (dialog_yesNo("The file " + doc.file.displayPath +
                         " does not exist.  Do you want to create it?") == "No") {
            return null;
        } else {
            var sysUtils = Components.classes["@activestate.com/koSysUtils;1"]
                            .createInstance(Components.interfaces.koISysUtils);
            sysUtils.Touch(doc.file.displayPath);
        }
    }
    if (doc.file.isDirectory) {
        dialog_alert('Komodo cannot open directories: ['+doc.file.path+']\n');
        return null;
    }
    try {
        if (doc.haveAutoSave() &&
            dialog_yesNo("It appears the file '"+doc.file.displayPath+
                        "' was not properly saved, would you "+
                        "like to restore the backup?", "Yes") == "Yes") {
            doc.restoreAutoSave();
        } else {
            doc.load();
        }
        doc.removeAutoSaveFile();
        // the following line is delayed to avoid notifications during load()
        if (viewList) {
            view = viewList.createViewFromDocument(doc,viewType);
        } else {
            view = this.topView.createViewFromDocument(doc,viewType);
        }
    } catch (e)  {
        var err = this.lastErrorSvc.getLastErrorMessage();
        dialog_alert('Komodo was unable to open the file: '+doc.file.baseName, err, 'File Open Error');
        this.log.exception(e);
        view = null;
    }
    gTimeline.leave('viewManager.newViewFromURI');
    this.log.info("leaving newViewFromURI");
    return view;
}

viewManager.prototype.doFileOpen = function(uri, viewType/*='editor'*/, viewList/*=null*/)
{
    uri = uriparse_fixupURI(uri);
    var views = this.topView.findViewsForURI(uri);
    if (views.length > 0) {
        views[0].makeCurrent();
        return views[0];
    }
    if (typeof(viewList)=='undefined')
        viewList = null;
    if (typeof(viewType)=='undefined' || !viewType)
        viewType = 'editor';
    return this.newViewFromURI(uri, viewType, viewList);
}

viewManager.prototype.doFileOpenAtLine = function(uri, lineno, viewType/*='editor'*/, viewList/*=null*/) {
    if (typeof(viewType)=='undefined' || !viewType)
        viewType = 'editor';
    if (typeof(viewList)=='undefined')
        viewList = null;
    var v = this.doFileOpen(uri, viewType, viewList);
    if (v) {
        v.currentLine = lineno;
    }
    return v;
}

viewManager.prototype.getViewForFile = function(uri) {
    var v = this.topView.findViewsForURI(uri);
    if (v.length > 0) return v[0];
    return null;
}

// XXX temporary hack for browser/iframe
viewManager.prototype.loadViewFromURI = function(uri, viewType/*='editor'*/)
{
    this.log.info("doing loadViewFromURI: " + uri);
    gTimeline.enter('viewManager.loadViewFromURI');
    if (typeof(viewType)=='undefined' || !viewType)
        viewType = 'editor';
    var doc = this.docSvc.createDocumentFromURI(uri);

    // the following line is delayed to avoid notifications during load()
    view = this.topView.createViewFromDocument(doc,viewType);
    gTimeline.leave('viewManager.loadViewFromURI');
    this.log.info("leaving loadViewFromURI");    
}

viewManager.prototype.cacheCommandData = function(view)
{
    var cache = new Object();
    cache.type = null;
    cache.sccSummary = null;
    cache.hasSelection = false;
    cache.isDirty = false;
    cache.canUndo = false;
    cache.canRedo = false;
    cache.canFold = false;
    cache.canDebug = false;
    cache.language = null;
    cache.canPreview = false;
    if (view) {
        cache.type = view.getAttribute('type');
        if (view.document) {
            cache.sccSummary = view.document.isUntitled?'':view.document.file.sccSummary;
            cache.hasSelection = view.selection != '';
            cache.isDirty = view.document.isDirty
            if (cache.type == 'editor') {
                cache.canUndo = view.scintilla.canUndo();
                cache.canRedo = view.scintilla.canRedo();
                cache.canPreview = view.document.file && view.document.file.isLocal;
            }
            if (view.document.languageObj) {
                cache.canFold = view.document.languageObj.foldable;
                cache.canDebug = view.document.languageObj.debuggable;
                cache.language = view.document.language;
            }
        }
    }
    return cache
}

viewManager.prototype.observe = function(subject, topic, data)
{
    this.log.debug("_ViewObserver: observed '"+topic+"' notification: data='"+data+"'\n");
    switch (topic) {
        case 'SciMoz:FileDrop':
        case 'open_file':
            this.log.info("got open_file notification: " + data);
            gTimeline.mark('opening file');
            var uri;
            try {
                uri = uriparse_localPathToURI(data);
            } catch(ex) {
                // Maybe it is a URI already?
                uri = data;
            }
            open_openURI(uri); // dispatch to view, project, UI builder as necessary
            gTimeline.mark('file opened');
            break;
        case 'current_view_changed':
            // Update the currentView
            // This has to happen always since one can do ctrl+tab,ctrl+i, and that
            // needs to find the right view.
            this.currentView = subject;
            if (this.batchMode) {
                // break out early -- we don't want to update controllers at this point.
                break;
            }
            if (this.topView.viewhistory.inBufferSwitchingSession) {
                // break out early -- we don't want to update controllers at this point.
                break;
            }
            oldcache = this.lastviewcache;
            newcache = this.cacheCommandData(this.currentView);
            //for (var x in oldcache) {
            //    dump('oldcache['+ x +'] = ' + oldcache[x] + '\n');
            //}
            //for (var x in newcache) {
            //    dump('newcache['+ x +'] = ' + newcache[x] + '\n');
            //}
            window.setTimeout("window.updateCommands('current_view_changed');", 1);
            var update_editor_change = (oldcache.type != newcache.type) &&
                (oldcache.type == 'editor' || newcache.type== 'editor');
            if (update_editor_change) {
                window.setTimeout("window.updateCommands('currentview_is_editor');", 1)
            }
            if (update_editor_change || oldcache.sccSummary!= newcache.sccSummary) {
                window.setTimeout("window.updateCommands('SCC');", 1);
            }
            if (update_editor_change || oldcache.isDirty != newcache.isDirty) {
                window.setTimeout("window.updateCommands('dirty');", 1);
            }
            if (update_editor_change || oldcache.canUndo != newcache.canUndo ||
                oldcache.canRedo != newcache.canRedo) {
                window.setTimeout("window.updateCommands('undo');", 1);
            }
            if (update_editor_change || oldcache.language != newcache.language) {
                window.setTimeout("window.updateCommands('language_changed');", 1)
            }
            if (update_editor_change || oldcache.canDebug != newcache.canDebug) {
                window.setTimeout("window.updateCommands('debuggability_changed');", 1);
            }
            if (update_editor_change || oldcache.canFold != newcache.canFold) {
                window.setTimeout("window.updateCommands('foldability_changed');", 1);
            }
            if (update_editor_change || oldcache.canPreview != newcache.canPreview) {
                window.setTimeout("window.updateCommands('previewability_changed');",1);
            }
            if (update_editor_change || oldcache.hasSelection != newcache.hasSelection) {
                window.setTimeout("window.updateCommands('select');", 1);
            }
            this.lastviewcache = newcache;
            break;
        case 'view_opened':
            this.log.info("got 'view opened' notification");
            this._viewCount++;
            this.log.info("_viewcount is " + this._viewCount);
            if (this._viewCount == 1) {
                this.log.info("sending event: 'some_files_open'");
                window.setTimeout("window.updateCommands('some_files_open');", 1);
            } else if (this._viewCount == 2) {
                // We've opened our second view
                window.setTimeout("window.updateCommands('second_view_open_close');", 1);
            }
            break;
        case 'view_closed':
            this._viewCount--;
            this.log.info("_viewcount is " + this._viewCount);
            if (this._viewCount == 0) {
                this.log.info("sending event: 'some_files_open'");
                window.setTimeout("window.updateCommands('some_files_open');", 1);
            } else if (this._viewCount == 1) {
                // We've closed our second view
                window.setTimeout("window.updateCommands('second_view_open_close');", 1);
            }
            break;
        case 'file_status':
            var views = this.topView.findViewsForURI(data);
            for (i=0; i < views.length; ++i) {
                // XXX optimize all this stuff
                views[i].updateFileStatus();
                views[i].updateDirtyStatus();
                views[i].item.updateStatus();
                if (views[i] == this.currentView) {
                    window.setTimeout("window.updateCommands('SCC');", 1);
                }
            }
            break;












    }
}

viewManager.prototype.supportsCommand = function(command) {
    if (command.indexOf("cmd_viewAs") == 0) {
        return true;
    }

    return Controller.prototype.supportsCommand.apply(this, [command]);
}

viewManager.prototype.isCommandEnabled = function(command) {
    if (command.indexOf("cmd_viewAs") == 0) {
        // XXX we want to use a broadcaster for this, there are too many.
        rc = gViewMgr.currentView && gViewMgr.currentView.getAttribute('type') == 'editor'
        //this.log.warn('asked about' + command + ', returning ' + rc );
        return rc;
    }

    return Controller.prototype.isCommandEnabled.apply(this, [command]);
}

viewManager.prototype.doCommand = function(command) {
    Controller.prototype.doCommand.apply(this, [command]);
}

viewManager.prototype.is_cmd_bufferClose_supported = function() {
    return 1;
}

viewManager.prototype.is_cmd_bufferClose_enabled = function() {
    return this.currentView;
}

viewManager.prototype.do_cmd_bufferClose = function() {
    this.currentView.close();
}

viewManager.prototype.is_cmd_closeAll_supported = function() {
    this.log.info('is_cmd_closeAll_supported');
    return 1;
}

viewManager.prototype.is_cmd_closeAll_enabled = function() {
    this.log.info('is_cmd_closeAll_enabled' + String(this._viewCount != 0));
    return this._viewCount != 0;
}

viewManager.prototype.do_cmd_closeAll = function() {
    this._doCloseAll();
}

viewManager.prototype._doCloseAll = function() {
    // returns true if all views were closed.
    var views = this.topView.getDocumentViews(true);
    var i;
    for (i = views.length-1; i >= 0; i--) {
        if (! views[i].close()) {
            return false;
        }
    }
    return true;
}

function _elementInArray(element, array)
{
    for (var i = 0; i < array.length; i++) {
        if (array[i] == element) {
            return true;
        }
    }
    return false;
}

viewManager.prototype.offerToSave = function(urls, /* default is null meaning all dirty files */
                                             title, /* default is "Save modified files?" */
                                             prompt, /* default is "Please select the files you wish to save:" */
                                             doNotAskPref, /* default is null */
                                             skipProjects /* default is false */
                                             ) {
    // This function offers to save the subset of 'urls' which correspond to dirty
    // files, whether corresponding to views, project files, or GUI dialogs.
    //
    // If 'urls' is null or unspecified, then it offers to save all dirty documents.
    //
    // This function returns false if the user cancels the operation, false otherwise.
    //
    // the 'doNotAskPref' can be used to let the user skip the dialog.  use carefully!
    //
    // If 'skipProjects' is true, then projects aren't considered
    //
    // Note that offerToSave needs to save all views that need saving, but actually close
    // none of them, in case another canclose handler cancels the close operation
    //
    // Note that this function is used in many places throughout the chrome.

    if (typeof(urls) == 'undefined') {
        urls = null;
    }
    if (typeof(title) == 'undefined') {
        title = "Save Modified Files?";
    }
    if (typeof(prompt) == 'undefined') {
        title = "Please select the files you wish to save:";
    }
    if (typeof(doNotAskPref) == 'undefined') {
        doNotAskPref = null;
    }
    if (typeof(skipProjects) == 'undefined') {
        skipProjects = false;
    }
    
    var views = this.topView.getViews(true);
    var i, view, item, k;
    var dirtyItems = [];
    var sofar = {};
    for (i = 0; i < views.length; i++) {
        view = views[i];
        if (!view) continue;
        // Persist view state now, to not have to do it as part of onunload handling
        if (view.getAttribute('type') == 'editor') {
            try {
                view.saveState();
            } catch (e) {
                this.log.error(e);
            }
        }
        if (typeof(view.isDirty) != 'undefined' && view.isDirty) {
            if (urls != null) {
                // Exclude the view if it's not in the urls list
                // There is no way that untitled documents could be in the list
                if (view.document.isUntitled) continue;
                // exclude if it's not in the list
                if (! _elementInArray(view.document.file.URI, urls)) continue;
                // exclude if we've already got it!
            }
            // We need to deal with views that are split and that share a document
            // So we keep track of the displayPaths, and only add a view
            // if it's display path isn't already in our list.
            if (view.document.displayPath in sofar) continue;
            sofar[view.document.displayPath] = true;
            item = new Object();
            item.type = 'view';
            item.view = view;
            dirtyItems.push(item);
        }
    }
    if (!skipProjects) {
        var dirtyProjectURLs = gProjectManager.getDirtyProjectURLs();
        for (i = 0; i < dirtyProjectURLs.length; i++) {
            if (urls != null && ! _elementInArray(dirtyProjectURLs[i], urls)) continue;
            item = new Object();
            item.type = 'project';
            item.URL = dirtyProjectURLs[i];
            dirtyItems.push(item);
        }
    }














    if (dirtyItems.length == 0) return true; // nothing to save
    function stringifier(item) {
        if (item.type == 'view') {
            return item.view.document.displayPath;
        } else {
            return uriparse_displayPath(item.URL);
        }
    }
    var itemsToSave = dialog_selectFromList(title,
                                            prompt,
                          dirtyItems,
                          "zero-or-more",
                          stringifier,
                          doNotAskPref,
                          true /* yesNoCancel */ );
    if (itemsToSave == null) return false; // canceled
    for (i = 0; i < itemsToSave.length; i++) {
        item = itemsToSave[i];
        if (item.type == 'view') {
            if (item.view.save() == false) {
                // the user hit cancel
                return false;
            }
        } else if (item.type == 'project') {
            var projectId = gProjectManager.getProjectIdByURL(item.URL);
            if (gProjectManager.saveProject(projectId) == false) {
                // the user hit cancel
                return false;
            }






        } else {
            this.log.error("Unknown item type in canClose handler");
        }
    }
        
    // remove any autosave files that might exist
    for (i = 0; i < dirtyItems.length; i++) {
        item = dirtyItems[i];
        if (item.view && item.view.document)
            item.view.document.removeAutoSaveFile();
    }
    return true;
}

viewManager.prototype.revertViewsByURL = function(urls) {
    var i, j, view, views;
    for (i = 0; i < urls.length; i++) {
        views = this.topView.findViewsForURI(urls[i]);
        for (j = 0; j < views.length; j++) {
            view = views[j];
            view.revertUnconditionally();
            view.document.removeAutoSaveFile();
        }
    }
}

// cmd_find
viewManager.prototype.is_cmd_find_supported = function() {
    return 1;
}

viewManager.prototype.is_cmd_find_enabled = function() {
    var currView = gViewMgr.currentView;
    if (currView && currView.getAttribute("type") == "editor") {
        return true;
    } else {
        return false;
    }
}

viewManager.prototype.do_cmd_find = function() {
    launch_Find();
}

// cmd_replace
viewManager.prototype.is_cmd_replace_supported = function() {
    return 1;
}

viewManager.prototype.is_cmd_replace_enabled = function() {
    return this._viewCount != 0;
}

viewManager.prototype.do_cmd_replace = function() {
    launch_Replace();
}

// cmd_findNext
viewManager.prototype.is_cmd_findNext_supported = function() {
    return true;
}

viewManager.prototype.is_cmd_findNext_enabled = function() {
    return gViewMgr.currentView ? true : false;
}

viewManager.prototype.do_cmd_findNext = function() {
    if (findSvc == null) {
        findSvc = Components.classes["@activestate.com/koFindService;1"].
                  getService(Components.interfaces.koIFindService);
    }
    var pattern = MRU_get("find-patternMru");
    if (pattern) {
        var context = Components.classes[
            "@activestate.com/koFindContext;1"]
            .createInstance(Components.interfaces.koIFindContext);
        context.type = findSvc.options.preferredContextType;
        Find_FindNext(window, context, pattern);
    } else {
        launch_Find();
    }
}

// cmd_findPrevious
viewManager.prototype.is_cmd_findPrevious_supported = function() {
    return true;
}

viewManager.prototype.is_cmd_findPrevious_enabled = function() {
    return gViewMgr.currentView ? true : false;
}

viewManager.prototype.do_cmd_findPrevious = function() {
    if (findSvc == null) {
        findSvc = Components.classes["@activestate.com/koFindService;1"].
                  getService(Components.interfaces.koIFindService);
    }
    var pattern = MRU_get("find-patternMru");
    if (pattern) {
        var context = Components.classes["@activestate.com/koFindContext;1"]
                      .createInstance(Components.interfaces.koIFindContext);
        context.type = findSvc.options.preferredContextType;
        // go in the opposite direction from the current setting
        findSvc.options.searchBackward = !findSvc.options.searchBackward;
        Find_FindNext(window, context, pattern);
        // restore the direction setting
        findSvc.options.searchBackward = !findSvc.options.searchBackward;
    } else {
        launch_Find();
    }
}

// cmd_findNextResult
viewManager.prototype.is_cmd_findNextResult_supported = function() {
    return true;
}

viewManager.prototype.is_cmd_findNextResult_enabled = function() {
    return gViewMgr.currentView ? true : false;
}

viewManager.prototype.do_cmd_findNextResult = function() {
    FindResultsTab_NextResult();
}

// cmd_findNextFunction, cmd_findPreviousFunction, cmd_findAllFunctions

// Auxiliary function used by the find*Function(s) methods.
//
//  "searchType" is one of "next", "previous" or "all"
//
viewManager.prototype._findFunction = function(searchType) { 
    try {
        var language = gViewMgr.currentView.document.languageObj;
        var re = language.namedBlockRE;
        var namedBlockDescription = language.namedBlockDescription;
        if (re == null || re == '')
            return;

        if (findSvc == null) {
            findSvc = Components.classes["@activestate.com/koFindService;1"].
                      getService(Components.interfaces.koIFindService);
        }
        var patternType, caseSensitivity, searchBackward, matchWord;
        patternType = findSvc.options.patternType;
        caseSensitivity = findSvc.options.caseSensitivity;
        searchBackward = findSvc.options.searchBackward;
        matchWord = findSvc.options.matchWord;
        
        findSvc.options.patternType = findSvc.options.FOT_REGEX_PYTHON;
        findSvc.options.caseSensitivity = findSvc.options.FOT_SENSITIVE;
        if (searchType == "previous") {
            findSvc.options.searchBackward = 1;
        } else {
            findSvc.options.searchBackward = 0;
        }
        findSvc.options.matchWord = 0;
 
        var context = Components.classes["@activestate.com/koFindContext;1"]
                      .createInstance(Components.interfaces.koIFindContext);
        context.type = Components.interfaces.koIFindContext.FCT_CURRENT_DOC;
        context.name = "the current document";
        if (searchType == "all") {
            Find_FindAll(window, context, re, namedBlockDescription);
        } else {
            Find_FindNext(window, context, re);
        }
        findSvc.options.patternType = patternType;
        findSvc.options.caseSensitivity = caseSensitivity;
        findSvc.options.searchBackward = searchBackward;
        findSvc.options.matchWord = matchWord;
    }  catch (e) {
        this.log.error(e);
    }
}

viewManager.prototype.is_cmd_findNextFunction_supported = function() {
    return true;
}

viewManager.prototype.is_cmd_findNextFunction_enabled = function() {
    view = gViewMgr.currentView
    return view != null && view.document && view.document.languageObj.namedBlockRE != '';
}

viewManager.prototype.do_cmd_findNextFunction = function() {
    this._findFunction("next");
}

viewManager.prototype.is_cmd_findPreviousFunction_supported = function() {
    return true;
}

viewManager.prototype.is_cmd_findPreviousFunction_enabled = function() {
    view = gViewMgr.currentView
    return view != null && view.document && view.document.languageObj.namedBlockRE != '';
}

viewManager.prototype.do_cmd_findPreviousFunction = function() {
    this._findFunction("previous");
}

viewManager.prototype.is_cmd_findAllFunctions_supported = function() {
    return true;
}

viewManager.prototype.is_cmd_findAllFunctions_enabled = function() {
    view = gViewMgr.currentView
    return view != null && view.document && view.document.languageObj.namedBlockRE != '';
}

viewManager.prototype.do_cmd_findAllFunctions = function() {
    this._findFunction("all");
}


// cmd_viewAsLanguage

viewManager.prototype.is_cmd_viewAsLanguage_supported = function() {
    return true;
}

viewManager.prototype.is_cmd_viewAsLanguage_enabled = function() {
    var currView = gViewMgr.currentView;
    if (currView && currView.getAttribute("type") == "editor") {
        return true;
    } else {
        return false;
    }
}


// cmd_gotoLine

viewManager.prototype.is_cmd_gotoLine_supported = function() {
    return true;
}

viewManager.prototype.is_cmd_gotoLine_enabled = function() {
    var currView = gViewMgr.currentView;
    if (currView && currView.getAttribute("type") == "editor") {
        return true;
    } else {
        return false;
    }
}

viewManager.prototype.do_cmd_gotoLine = function() {
    function parseLine(line) {
        // Return [sign, num] for the given line string.
        //   42     sign=null, num=42
        //   +1     sign=+, num=1
        //   a      sign=null, num=NaN
        var stripped = line.replace(/(^\s*|\s*$)/g, '');
        var sign = null, num;
        if (stripped[0] == "-" || stripped[0] == "+") {
            sign = stripped[0];
            num = parseInt(stripped.substring(1, stripped.length));
        } else {
            sign = null;
            num = parseInt(stripped);
        }
        //dump("parseLine(line="+line+"): sign="+sign+", num="+num+"\n");
        return [sign, num];
    }
    
    function validateLine(line) {
        // Good: 1, 42, +12, -5. Bad: a, +1a, -, 3.14
        var parsed = parseLine(line);
        var sign = parsed[0];
        var num = parsed[1];
        //dump("validateLine(line="+line+"): sign="+sign+", num="+num+"\n");
        if (isNaN(num) || num < 0) {
            alert("'"+line+"' is invalid. You must enter a number, with "+
                  "an optional '+' or '-' prefix.");
            return false;
        }
        return true;
    }
    
    // Prompt for the line.
    var view = gViewMgr.currentView;
    var line = dialog_prompt(
            // prompt
            "Enter a line number to go to. Prefix + or - to move relative "+
            "to the current line. For example: +4 will move forward 4 lines.",
            "Enter line number:", // label
            null, // value
            "Go To Line", // title
            // mruName: Don't use one because this tends to gobble up one
            // <Enter> keypress, which is annoying more than the MRU is
            // potentially useful. Use the following to get per-file-mru:
            //      "goto_line_"+view.document.displayPath,
            null,
            validateLine) //validator

    if (line == null)
        return;  // dialog cancelled
    
    // Go to the given line.
    var parsed = parseLine(line);
    var sign = parsed[0];
    var num = parsed[1];
    var scimoz = view.scintilla.scimoz;
    var currLine;
    switch (sign) {
    case null:
        scimoz.gotoLine(num-1);  // scimoz handles out of bounds
        break;
    case "+":
        currLine = scimoz.lineFromPosition(scimoz.currentPos);
        scimoz.gotoLine(currLine + num);
        break;
    case "-":
        currLine = scimoz.lineFromPosition(scimoz.currentPos);
        scimoz.gotoLine(currLine - num);
        break;
    default:
        log.error("unexpected goto line 'sign' value: '"+sign+"'")
        break;
    }
}


// cmd_save
viewManager.prototype.is_cmd_save_supported = function() {
    return this.currentView != null && typeof(this.currentView.save) != 'undefined';
}

viewManager.prototype.is_cmd_save_enabled = function() {
    return this.currentView && this.currentView.document && this.currentView.document.isDirty;
}


viewManager.prototype.do_cmd_save = function() {
    this.currentView.save();
}


// cmd_revert
viewManager.prototype.is_cmd_revert_supported = function() {
    return this.currentView != null && typeof(this.currentView.revert) != 'undefined';
}

viewManager.prototype.is_cmd_revert_enabled = function() {
    return this.currentView && this.currentView.document && this.currentView.document.isDirty;
}


viewManager.prototype.do_cmd_revert = function() {
    this.currentView.revert();
}

// cmd_saveAll
viewManager.prototype.is_cmd_saveAll_supported = function() {
    return true;
}

viewManager.prototype.is_cmd_saveAll_enabled = function() {
    return true;
}

viewManager.prototype.do_cmd_saveAll = function() {
    var views = this.topView.getViews(true);
    var i, view;
    this.log.info("length of views is: " + views.length);
    for (i = views.length-1; i >= 0; i--) {
        view = views[i];
        if (view.document && view.document.isDirty) {
            view.save(); // we'll ignore return values here.
        }
    }

    // Save the toolbox data
    try {
        toolboxMgr.toolbox.save();
    } catch(ex) {
        dialog_alert('There was an error saving the toolbox: ', this.lastErrorSvc.getLastErrorMessage(), 'Toolbox Save Error');
    }











    // Save all dirty projects
    var projects = gProjectManager.getDirtyProjectIDs();
    for (i = 0; i < projects.length; i++) {
        gProjectManager.saveProject(projects[i]);
    }

    // Save prefs
    gPrefSvc.saveState();

    // save workspace
    view_saveWorkspace();
}


// cmd_saveAs
viewManager.prototype.is_cmd_saveAs_supported = function() {
    return this.currentView != null && typeof(this.currentView.saveAs) != 'undefined';
}

viewManager.prototype.is_cmd_saveAs_enabled = function() {
    return (this.currentView && this.currentView.document &&
            this.currentView.getAttribute('type') == 'editor');
}

viewManager.prototype.do_cmd_saveAs = function() {
    this.currentView.saveAs();
}

// cmd_saveAs_remote
viewManager.prototype.is_cmd_saveAs_remote_supported = function() {
    return this.currentView != null && typeof(this.currentView.saveAsRemote) != 'undefined';
}

viewManager.prototype.is_cmd_saveAs_remote_enabled = function() {
    return (this.currentView && this.currentView.document &&
            this.currentView.getAttribute('type') == 'editor');
}

viewManager.prototype.do_cmd_saveAs_remote = function() {
    this.currentView.saveAsRemote();
}

// cmd_bufferNext
viewManager.prototype.is_cmd_bufferNext_supported = function() {
    return 1;
}

viewManager.prototype.is_cmd_bufferNext_enabled = function() {
    return this._viewCount > 1;
}

viewManager.prototype.do_cmd_bufferNext = function() {
    this.topView.makeNextViewCurrent();
}

// cmd_bufferPrevious
viewManager.prototype.is_cmd_bufferPrevious_supported = function() {
    return 1;
}

viewManager.prototype.is_cmd_bufferPrevious_enabled = function() {
    return this._viewCount > 1;
}

viewManager.prototype.do_cmd_bufferPrevious = function() {
    this.topView.makePreviousViewCurrent();
}

// cmd_bufferNextMostRecent
viewManager.prototype.is_cmd_bufferNextMostRecent_enabled = function() {
    return this._viewCount > 1;
}

viewManager.prototype.do_cmd_bufferNextMostRecent = function() {
    this.topView.viewhistory.doNextMostRecentView(this.currentView);
}

viewManager.prototype.is_cmd_bufferNextLeastRecent_enabled = function() {
    return this._viewCount > 1;
}

// cmd_bufferLeastMostRecent
viewManager.prototype.do_cmd_bufferNextLeastRecent = function() {
    this.topView.viewhistory.doNextLeastRecentView(this.currentView);
}

// cmd_splittab
viewManager.prototype.is_cmd_splittab_supported = function() {
    return 1;
}

viewManager.prototype.is_cmd_splittab_enabled = function() {
    return (this.currentView && this.currentView.document &&
        this.topView.canSplitView(this.currentView));
}

viewManager.prototype.do_cmd_splittab = function() {
    this.topView.splitView(this.currentView);
}

// cmd_movetab
viewManager.prototype.is_cmd_movetab_supported = function() {
    return 1;
}

viewManager.prototype.is_cmd_movetab_enabled = function() {
    return this._viewCount > 1 &&
        this.currentView && this.currentView.document &&
        this.topView.canMoveView(this.currentView);
}

viewManager.prototype.do_cmd_movetab = function() {
    this.topView.moveView(this.currentView);
}

// cmd_movetab
viewManager.prototype.is_cmd_rotateSplitter_supported = function() {
    return 1;
}

viewManager.prototype.is_cmd_rotateSplitter_enabled = function() {
    var othertabbox = document.getElementById('view-2');
    return (!othertabbox.hasAttribute('collapsed') ||
            othertabbox.getAttribute('collapsed') == 'false');
}

viewManager.prototype.do_cmd_rotateSplitter = function() {
    try {
        this.topView.changeOrient();
    } catch (e) {
        log.exception(e);
    }
}
// cmd_cancel
viewManager.prototype.is_cmd_cancel_supported = function() {
    return 1;
}

viewManager.prototype.is_cmd_cancel_enabled = function() {
    return this.currentView && this.currentView.getAttribute('type') == 'editor';
}

viewManager.prototype.do_cmd_cancel = function() {
    if (!this.currentView || this.currentView.getAttribute('type') != 'editor') {
        return;
    }
    sm = this.currentView.scimoz
    if (sm.callTipActive()) {
        sm.callTipCancel()
    } else if (sm.autoCActive()) {
        sm.autoCCancel()
    }
}

viewManager.prototype.is_cmd_print_supported = function() {
    return 1;
}

viewManager.prototype.is_cmd_print_enabled = function() {
    if (!this.currentView) return false;
    var type = this.currentView.getAttribute('type');
    switch (type) {
        case 'editor':
        //case 'browser':
            return true;
        default:
            return false
    }
}

viewManager.prototype.do_cmd_print = function() {
    printing_print(this.currentView, 0, 0);
}

viewManager.prototype.is_cmd_printPreview_supported = function() {
    return 1;
}

viewManager.prototype.is_cmd_printPreview_enabled = function() {
    if (!this.currentView) return false;
    var type = this.currentView.getAttribute('type');
    switch (type) {
        case 'editor':
        //case 'browser':
            return true;
        default:
            return false
    }
}

viewManager.prototype.do_cmd_printPreview = function() {
    printing_print(this.currentView, 1, 0);
}

viewManager.prototype.is_cmd_exportHTML_supported = function() {
    return 1;
}

viewManager.prototype.is_cmd_exportHTML_enabled = function() {
    return this.currentView && this.currentView.getAttribute('type') == 'editor';
}

viewManager.prototype.do_cmd_exportHTML = function() {
    printing_print(this.currentView, 0, 1);
}

viewManager.prototype.do_ViewAs = function(language) {
    this.currentView.document.language = language;
}
//    id="cmd_exportHTML"
//    id="cmd_saveAs_remote"
//    id="cmd_saveAsTemplate"

viewManager.prototype.is_cmd_editPrefsCurrent_enabled = function () {
    return this.currentView && this.currentView.getAttribute('type') == 'editor';
}
viewManager.prototype.do_cmd_editPrefsCurrent = function () {
    try {
        var obj = new Object();
        obj.view = gViewMgr.currentView;
        window.openDialog("chrome://jaguar/content/editor/currentViewPrefs.xul", 
                          "komodo_viewprefs",
                          "chrome,scrollbars,dialog=yes,close,modal=yes",
                          obj);
        
        if (obj.retval == "Cancel")
            return;
    
    } catch(ex) {
        this.log.exception(ex, "Error editting current view prefs.");
    }
}

viewManager.prototype.is_cmd_saveAsTemplate_enabled = function () {
    return this.currentView && this.currentView.getAttribute('type') == 'editor';
}
viewManager.prototype.do_cmd_saveAsTemplate = function () {
    try {
        var obj = new Object();
        obj.view = gViewMgr.currentView;
        var os = Components.classes["@activestate.com/koOs;1"].getService();
        var templateSvc = Components.classes["@activestate.com/koTemplateService;1"].getService();
        var dname = os.path.join(templateSvc.getUserTemplatesDir(), 'My Templates'); 
        var templatename = filepicker_saveFile(dname, obj.view.document.baseName);
        if (!templatename) return;
        var docsvc = Components.classes['@activestate.com/koDocumentService;1']
                    .getService(Components.interfaces.koIDocumentService);
        var doc = docsvc.createDocumentFromURI(uriparse_localPathToURI(templatename));
        doc.buffer = view.document.buffer;
        doc.encoding = view.document.encoding;
        doc.save(0);
    } catch(ex) {
        this.log.exception(ex, "Error editting current view prefs.");
    }
}


function view_openFilesWithPicker(viewType/*='editor'*/) {
    if (typeof(viewType)=='undefined' || !viewType) viewType = 'editor';

    // We want the default directory to be that of the current file if there is one
    var defaultDir = null;
    if (gViewMgr.currentView &&
        gViewMgr.currentView.getAttribute("type") == "editor" &&
        gViewMgr.currentView.document &&
        !gViewMgr.currentView.document.isUntitled) {
        defaultDir = gViewMgr.currentView.document.file.dirName;
    }

    var paths = filepicker_openFiles(defaultDir);
    if (paths == null)
        return;
    open_openMultipleURIs(paths, viewType);
}

function view_openTemplatesWithPicker(viewType/*='editor'*/) {
    try {
        if (typeof(viewType)=='undefined' || !viewType) viewType = 'editor';
    
        var os = Components.classes["@activestate.com/koOs;1"].getService();
        var templateSvc = Components.classes["@activestate.com/koTemplateService;1"].getService();
        var defaultDir = templateSvc.getUserTemplatesDir();
        var paths = filepicker_openFiles(defaultDir);
        if (paths == null)
            return;
        open_openMultipleURIs(paths, viewType);
    } catch (e) {
        log.exception(e);
    }
}

function view_editorHasFocus()
{
    /* XXX the usage of this is to find out who has focus, but
       having a view of type editor is not a good indicator
       that the focus is on the editor.  We can always make
       sure that the focus is not in the project or toolbox
       but this is probably less efficient than we want. */
    return !view_projectHasFocus() &&
        !view_toolboxHasFocus() &&
        gViewMgr.currentView &&
        gViewMgr.currentView.getAttribute('type') == 'editor';
}

function view_projectHasFocus()
{
    return document.getElementById('projectview').focused;
}

function view_toolboxHasFocus()
{
    return document.getElementById('toolboxview').focused;
}

function view_restoreWorkspace()
{
    var infoSvc = Components.classes["@activestate.com/koInfoService;1"].getService();
    if (infoSvc.nonInteractiveMode) return;

    if (gPrefs.hasPref('workspace') &&
        dialog_yesNo("Do you want to open recent files and projects?",
                     null, null, null, "restore_workspace") == "Yes")
    {
        var workspace = gPrefs.getPref('workspace');
        var cnt = new Object();
        var ids = new Object();
        var id, elt;
        workspace.getPrefIds(ids, cnt);
        if (workspace.hasPref('opened_projects')) {
            pref = workspace.getPref('opened_projects');
            gProjectManager.setState(pref)
        }
        for (var i = 0; i < ids.value.length; i++) {
            id = ids.value[i];
            elt = document.getElementById(id);
            if (elt) {
                var pref = workspace.getPref(id);
                elt.setState(pref);
            }
        }
    }
    KomodoRegisterOnUnloadHandler(view_saveWorkspace, this);
}

function view_saveWorkspace()
{
    try {
        // Ask each major component to serialize itself to a pref.
        var workspace;
        if (gPrefs.hasPref('workspace')) {
            workspace = gPrefs.getPref('workspace');
        } else {
            workspace = Components.classes['@activestate.com/koPreferenceSet;1'].createInstance();
            workspace.id = 'workspace';
        }
        var haveStateToPersist = false;
        pref = gProjectManager.getState()
        if (pref) {
            workspace.setPref(pref.id, pref)
            haveStateToPersist = true;
        } else if (workspace.hasPref('opened_projects')) {
            workspace.deletePref('opened_projects');
        }
        var ids = ['topview'];
        var elt, id, pref;
        for (i = 0; i < ids.length; i++) {
            id = ids[i];
            elt = document.getElementById(id);
            if (!elt) {
                alert("couldn't find " + id );
            }
            pref = elt.getState();
            if (pref) {
                haveStateToPersist = true;
                pref.id = id;
                workspace.setPref(id, pref);
            } else if (workspace.hasPref(id)) {
                workspace.deletePref(id);
            }
        }
        if (haveStateToPersist) {
            gPrefs.setPref(workspace.id, workspace);
        } else if (gPrefs.hasPref(workspace.id)) {
            gPrefs.deletePref(workspace.id);        
        }
    } catch (e) {
        log.exception(e,"Error saving workspace: ");
    }
}

function _itemStringifier(item)
{
    return item.file.displayPath;
}

var _gInCheckDiskFiles = false;
// view_checkDiskFiles is only called from the window's focus handler,
// located in jaguar.xul
function view_checkDiskFiles(event)
{
    if (_gInCheckDiskFiles || gViewMgr.batchMode) {
        return true;
    }
    var checkDisk = (gPrefs.hasBooleanPref("checkDiskFile") &&
                     gPrefs.getBooleanPref("checkDiskFile"));
    if (!checkDisk) return true;
    _gInCheckDiskFiles = true;
    log.info('Checking Disk Files');
    window.setTimeout('_view_checkDiskFiles();',1);
    return true;
}

function _view_checkDiskFiles(event) {
    // checks open files and projects for dirtiness
    var fileSvc = Components.classes["@activestate.com/koFileService;1"].
                    getService();
    try {
        window.updateCommands('clipboard');
        var changedItems = [];
        var removedItems = [];
        var viewsToReload;
        var conflictedItems = [];
        var view, url, i, j, hasChanged;
        var views, file, prompt, title, item, items;

        // Deal with views first
        views = gViewMgr.topView.getDocumentViewList(true);
        for (i = 0; i < views.length; i++) {
            view = views[i];
            // browser views do not load via document, so will
            // always be wrong when checking hasChanged
            if (view.getAttribute('type')!='editor') continue;
            if (typeof(view.document) == 'undefined' ||
                !view.document ||
                view.document.isUntitled) continue;
            file = view.document.file;
            hasChanged = file.hasChanged;
            if (file.exists && hasChanged) {
                    gObserverSvc.notifyObservers(this,'file_changed',file.URI);
            }
            item = new Object;
            item.type = 'view';
            item.view = view;
            item.file = file;
            if (!file.exists) {
                removedItems.push(item);
            } else {
                if (view.document.differentOnDisk()) {
                    if (view.document.isDirty) {
                        conflictedItems.push(item);
                    } else {
                        changedItems.push(item);
                    }
                }
            }
        }

        // Now look for changed projects
        var projects = gProjectManager._projects;
        for (i = 0; i < projects.length; i++) {
            file = projects[i].file;
            if (file.hasChanged) {
                item = new Object;
                item.type = 'project';
                item.project = projects[i];
                item.file = file;
                if (!file.exists) {
                    removedItems.push(item);
                } else if (item.project.part.isDirty) {
                    conflictedItems.push(item);
                } else {
                    changedItems.push(item);
                }
            }
        }

        // handle files and projects that have changed on disk
        if (changedItems.length > 0) {
            title = 'Reload Changed Files and Projects';
            prompt = 'Some open files and/or projects have changed on '+
                     'disk, do you want to reload them?';
            items = dialog_selectFromList(title,
                                          prompt,
                                          changedItems,
                                          'zero-or-more',
                                          _itemStringifier,
                                          'reload_changed_files');
            if (items != null && items.length > 0) {
                for (i = 0; i < items.length; i++) {
                    if (item.type == 'view') {
                        items[i].view.revertUnconditionally()
                    } else {
                        gProjectManager.revertProjectByURL(items[i].project.url);
                    }
                }
            }
        }
        // handle files and projects that have changed on disk
        if (removedItems.length > 0) {
            for (i = 0; i < removedItems.length; ++i) {
                if (removedItems[i].view.document) {
                    removedItems[i].view.document.isDirty = true;
                }
            }
            title = 'Close Deleted Files and Projects';
            prompt = 'The following open files and/or projects have been '+
                     'deleted from disk.  Do you want to close them?';
            items = dialog_selectFromList(title,
                                          prompt,
                                          removedItems,
                                          'zero-or-more',
                                          _itemStringifier,
                                          'close_deleted_files');
            if (items != null && items.length > 0) {
                for (i = 0; i < items.length; i++) {
                    if (item.type == 'view') {
                        items[i].view.close()
                    } else {
                        gProjectManager.closeProjectByURL(items[i].project.url);
                    }
                }
            }
        }

        //XXX You can get this dialog when closing Komodo with
        //    a file one choses to not save and after everything else has
        //    closed so there is nothing that can really be done.
        // handle files and projects that have changed and are dirty
        if (conflictedItems.length > 0) {
            prompt = 'The following files and projects have changed on disk '+
                     'and are modified -- either revert them or save your changes:';
            title = 'Modified files have changed on disk';
            text = '';
            for (i = 0; i < conflictedItems.length; i++) {
                text += _itemStringifier(conflictedItems[i]) + '\n'
            }
            dialog_alert(prompt, text, title)
        }
    } catch(e) {
        log.exception(e);
    }
    gObserverSvc.notifyObservers(null,'file_update_now',url)
    _gInCheckDiskFiles = false;
    return true;
}

function view_onkeypresshandler(event)
{
    if (event.keyCode == event.DOM_VK_TAB && event.ctrlKey) {
        if (event.shiftKey) {
            command_doCommand('cmd_bufferNextLeastRecent')
        } else {
            command_doCommand('cmd_bufferNextMostRecent')
        }
        event.stopPropagation();
        event.preventDefault();
    }
}
