#!/usr/bin/env python
# Copyright (c) 2003 ActiveState Corp.
# See the file LICENSE.txt for licensing information.

import sys
from xpcom import components, nsError, ServerException, COMException



#---- component implementation

class KoMacroService:
    _com_interfaces_ = [components.interfaces.koIMacroService]
    _reg_clsid_ = "D6126643-84DC-4DDE-9101-C4BF9B40F588"
    _reg_contractid_ = "@activestate.com/koMacroService;1"
    _reg_desc_ = "Service for running Macros"

    def __init__(self):
        self.lastErrorSvc = components.classes["@activestate.com/koLastErrorService;1"].\
            getService(components.interfaces.koILastErrorService)
        self.wm = components.classes["@mozilla.org/appshell/window-mediator;1"].\
            getService(components.interfaces.nsIWindowMediator)
        self.dialogProxy = components.classes["@activestate.com/asDialogProxy;1"].\
            getService(components.interfaces.asIDialogProxy)


    def runString(self, language, code):
        if language == "python":
            try:
                exec(code)
            except:
                tb = ''.join(traceback.format_exception(*sys.exc_info()))
                self.lastErrorSvc.setLastError(nsError.NS_ERROR_FAILURE, tb)
                raise ServerException(nsError.NS_ERROR_FAILURE, tb)

        elif language == "javascript":
            window = self.wm.getMostRecentWindow(None)
            try:
                self.dialogProxy.eval_(window, code);
            except COMException, ex:
                # eval_() already set "last error".
                raise ServerException(nsError.NS_ERROR_FAILURE)

        else:
            msg = "Unknown macro language: '%s'" % language
            self.lastErrorSvc.setLastError(nsError.NS_ERROR_FAILURE, msg)
            raise ServerException(nsError.NS_ERROR_FAILURE, msg)


    def runFile(self, language, filename):
        if language == "python":
            try:
                execfile(filename)
            except:
                tb = ''.join(traceback.format_exception(*sys.exc_info()))
                self.lastErrorSvc.setLastError(nsError.NS_ERROR_FAILURE, tb)
                raise ServerException(nsError.NS_ERROR_FAILURE, tb)

        elif language == "javascript":
            try:
                fin = open(filename, 'r')
            except (IOError, OSError), ex:
                msg = str(ex)
                self.lastErrorSvc.setLastError(nsError.NS_ERROR_FAILURE, msg)
                raise ServerException(nsError.NS_ERROR_FAILURE, msg)
            try:
                code = fin.read()
            finally:
                fin.close()

            window = self.wm.getMostRecentWindow(None)
            try:
                self.dialogProxy.eval_(window, code);
            except COMException, ex:
                # eval_() already set "last error".
                raise ServerException(nsError.NS_ERROR_FAILURE)

        else:
            msg = "Unknown macro language: '%s'" % language
            self.lastErrorSvc.setLastError(nsError.NS_ERROR_FAILURE, msg)
            raise ServerException(nsError.NS_ERROR_FAILURE, msg)
            

