# Copyright (c) 2000-2003 ActiveState SRL.
# See the file LICENSE.txt for licensing information.

r"""Shim for pure Python importing of the true 'pywintypes' module.

The Problem:

The loading of pythoncomXX.dll and PyWinTypesXX.dll
    (a) via direct loading by of PyWin32's .pyd's; and
    (b) via imports from Python code
must get
    (1) Komodo's siloed versions of these DLLs; and
    (2) the same physical DLL on disk.


The Discussion and Solution:

To satisfy (a) these DLLs must either be in the same directory as those
PyWin32 .pyd's or in the system directory. The system directory is no
good because subsequent PyWin32/ActivePython installs might install a
binary-incompatible version of these DLLs (see
http://bugs.activestate.com/Komodo/show_bug.cgi?id=22523). Thankfully
all the PyWin32 .pyd's the Komodo currently uses are in the same
directory, namely:
    <ko-install-dir>\python\Lib\site-packages\win32
so we can put our DLLs there. (XXX Note: win32comext/... includes some
PyWin32 .pyds which Komodo *may* want to use in the future. If those
require PyWinTypesXY.dll then we may have to do some jimmying of PyWin32.)

This Python module is here to satisfy condition (b). It turns this code
    import pywintypes
into a proper loading of PyWinTypesXX.dll in the new install location we
have given for it.
"""

import os
import sys
import imp

dllDir = os.path.join(os.environ["KOMODO_INSTALLDIR"],
                      "python", "Lib", "site-packages", "win32")
major, minor = sys.version_info[:2]
imp.load_dynamic("pywintypes",
                 os.path.join(dllDir, "PyWinTypes%d%d.dll" % (major, minor)))

