"""Test Project functionality in Komodo."""

import os, sys
import unittest
from xpcom import components, ServerException, COMException, nsError
from xpcom.server import WrapObject, UnwrapObject
from convertURL import *


class TestKoFileSpec(unittest.TestCase):

    def __init__(self, foo):
        unittest.TestCase.__init__(self, foo)
        self.__fileSpec = \
                        components.classes["@activestate.com/koFileSpec;1"] \
                        .createInstance(components.interfaces.koIFileSpec)
        # we need to know the platform to determine which tests to run
        self.__infoService = components.classes["@activestate.com/koInfoService;1"] \
                       .getService(components.interfaces.koIInfoService)


    def test_assignAndRetrieveURL(self):
        url = "http://www.activestate.com"
        self.__fileSpec.url = url
        assert self.__fileSpec.url == url
 
    def test_assignAndRetrievePath(self):
        if self.__infoService.platform == "win32":
            path = r"C:\Documents and Settings\test\My Documents\somefile.txt"
        else:
            path = "/home/test/somefile.txt"

        self.__fileSpec.path = path
        assert self.__fileSpec.path == os.path.normpath(path)


    def test_determinePathFromFileURL(self):
        if self.__infoService.platform == "win32":
            url = "file:///C:/Documents%20and%20Settings/test/My%20Documents/somefile.txt"
            path = r"C:\Documents and Settings\test\My Documents\somefile.txt"
        else:
            url = "file:///home/test/somefile.txt"
            path = "/home/test/somefile.txt"

        self.__fileSpec.url = url
        assert self.__fileSpec.path == path


    def test_determineFileURLFromPath(self):
        if self.__infoService.platform == "win32":
            url = "file:///C:/Documents%20and%20Settings/test/My%20Documents/somefile.txt"
            path = r"C:\Documents and Settings\test\My Documents\somefile.txt"
        else:
            url = "file:///home/test/somefile.txt"
            path = "/home/test/somefile.txt"

        self.__fileSpec.path = path
        assert self.__fileSpec.url == url


    def test_assignAndRetrieveUNCPath(self):
        if self.__infoService.platform == "win32":
            path = r"\\americano\home\Apps\Komodo\somefile.txt"
            self.__fileSpec.path = path
            assert self.__fileSpec.path == path
        else:
            return


    def test_determineUNCPathFromFileURL(self):
        if self.__infoService.platform == "win32":
            path = r"\\americano\home\Apps\Komodo\somefile.txt"
            url = "file://///americano/home/Apps/Komodo/somefile.txt"
            self.__fileSpec.url = url
            assert self.__fileSpec.path == path
        else:
            return

    def test_determineFileURLFromUNCPath(self):
        if self.__infoService.platform == "win32":
            path = r"\\americano\home\Apps\Komodo\somefile.txt"
            url = "file://///americano/home/Apps/Komodo/somefile.txt"
            self.__fileSpec.path = path
            assert self.__fileSpec.url == url
        else:
            return 
            

#---- mainline

def suite():
    return unittest.makeSuite(TestKoFileSpec)

def test_main():
    runner = unittest.TextTestRunner(verbosity=2)
    runner.run(suite())

if __name__ == "__main__":
    test_main()


